@@ -107,7 +107,9 @@
 #include "scripting/flash/globalization/lastoperationstatus.h"
 #include "scripting/flash/globalization/localeid.h"
 #include "scripting/flash/globalization/currencyformatter.h"
+#include "scripting/flash/globalization/currencyparseresult.h"
 #include "scripting/flash/globalization/numberformatter.h"
+#include "scripting/flash/globalization/numberparseresult.h"
 #include "scripting/flash/globalization/stringtools.h"
 #include "scripting/flash/external/ExternalInterface.h"
 #include "scripting/flash/media/flashmedia.h"
