@@ -236,7 +236,9 @@ REGISTER_CLASS_NAME(DateTimeStyle,"flash.globalization")
 REGISTER_CLASS_NAME(LastOperationStatus,"flash.globalization")
 REGISTER_CLASS_NAME(LocaleID,"flash.globalization")
 REGISTER_CLASS_NAME(NumberFormatter,"flash.globalization")
+REGISTER_CLASS_NAME(NumberParseResult,"flash.globalization")
 REGISTER_CLASS_NAME(CurrencyFormatter,"flash.globalization")
+REGISTER_CLASS_NAME(CurrencyParseResult,"flash.globalization")
 REGISTER_CLASS_NAME(StringTools,"flash.globalization")
 REGISTER_CLASS_NAME(CollatorMode,"flash.globalization")
 REGISTER_CLASS_NAME(DateTimeNameContext,"flash.globalization")
@@ -452,4 +454,7 @@ REGISTER_CLASS_NAME2(AVM1Video,"Video","")
 REGISTER_CLASS_NAME2(AVM1Color,"Color","")
 REGISTER_CLASS_NAME2(AVM1Broadcaster,"AsBroadcaster","")
 REGISTER_CLASS_NAME2(AVM1LocalConnection,"LocalConnection","")
+REGISTER_CLASS_NAME2(AVM1LoadVars,"LoadVars","")
 REGISTER_CLASS_NAME2(AVM1Bitmap,"Bitmap","")
+REGISTER_CLASS_NAME2(AVM1BitmapData,"BitmapData","")
+REGISTER_CLASS_NAME2(AVM1NetConnection,"NetConnection","")
