@@ -25,6 +25,7 @@
 
 namespace lightspark
 {
+class NetStream;
 
 class BuiltinStreamDecoder: public StreamDecoder
 {
@@ -38,15 +39,18 @@ private:
 	uint32_t decodedTime;
 	double frameRate;
 	ScriptDataTag metadataTag;
-	enum STREAM_TYPE { FLV_STREAM=0 };
+	enum STREAM_TYPE { FLV_STREAM=0, UNKOWN_STREAM=1 };
 	STREAM_TYPE classifyStream(std::istream& s);
+	NetStream* netstream;
+	uint8_t* headerbuf;
+	uint32_t headerLen;
 public:
-	BuiltinStreamDecoder(std::istream& _s);
-	bool decodeNextFrame();
-	bool getMetadataInteger(const char* name, uint32_t& ret) const;
-	bool getMetadataDouble(const char* name, double& ret) const;
+	BuiltinStreamDecoder(std::istream& _s, NetStream* _ns);
+	~BuiltinStreamDecoder();
+	bool decodeNextFrame() override;
+	void jumpToPosition(number_t position) override;
 };
 
-};
+}
 
 #endif /* BACKENDS_BUILTINDECODER_H */
