@@ -19,16 +19,17 @@
 
 #include <glib.h>
 #include <string>
-#include <boost/filesystem.hpp>
+#include <sys/stat.h>
 
 #include "backends/config.h"
 #include "compat.h"
 #include "logger.h"
 #include "exceptions.h"
 
+#include <cstring>
+
 using namespace lightspark;
 using namespace std;
-using namespace boost::filesystem;
 
 Config* Config::getConfig()
 {
@@ -77,14 +78,13 @@ std::string readRegistryEntry(std::string name)
 #endif
 
 Config::Config():
-	parser(NULL),
+	parser(nullptr),
 	//CONFIGURATION FILENAME AND SEARCH DIRECTORIES
 	configFilename("lightspark.conf"),
 	systemConfigDirectories(g_get_system_config_dirs()),userConfigDirectory(g_get_user_config_dir()),
 	//DEFAULT SETTINGS
-	defaultCacheDirectory((string) g_get_user_cache_dir() + "/lightspark"),
+	defaultCacheDirectory((string) g_get_user_cache_dir() + G_DIR_SEPARATOR_S + "lightspark"),
 	cacheDirectory(defaultCacheDirectory),cachePrefix("cache"),
-	audioBackend(INVALID),audioBackendName(""),
 	renderingEnabled(true)
 {
 #ifdef _WIN32
@@ -92,27 +92,24 @@ Config::Config():
 	if(exePath)
 		userConfigDirectory = exePath;
 #endif
-	audioBackendNames[PULSEAUDIO] = "pulseaudio";
-	audioBackendNames[SDL] = "sdl";
-	audioBackendNames[WINMM] = "winmm";
 
 	//Try system configs first
 	string sysDir;
 	const char* const* cursor = systemConfigDirectories;
-	while(*cursor != NULL)
+	while(*cursor != nullptr)
 	{
 		sysDir = *cursor;
-		parser = new ConfigParser(sysDir + "/" + configFilename);
+		parser = new ConfigParser(sysDir + G_DIR_SEPARATOR_S + configFilename);
 		while(parser->read())
 			handleEntry();
 		delete parser;
-		parser = NULL;
+		parser = nullptr;
 
 		++cursor;
 	}
 
 	//Try user config next
-	parser = new ConfigParser(userConfigDirectory + "/" + configFilename);
+	parser = new ConfigParser(userConfigDirectory + G_DIR_SEPARATOR_S + configFilename);
 	while(parser->read())
 		handleEntry();
 	delete parser;
@@ -125,33 +122,34 @@ Config::Config():
 #endif
 
 	//If cache dir doesn't exist, create it
-	path cacheDirectoryP(cacheDirectory);
-	if(!is_directory(cacheDirectoryP))
+	if (g_mkdir_with_parents(cacheDirectory.c_str(),S_IRUSR | S_IWUSR | S_IXUSR))
 	{
-		LOG(LOG_INFO, "Cache directory does not exist, trying to create");
-		try
-		{
-			create_directories(cacheDirectoryP);
-		}
-		catch(const filesystem_error& e)
-		{
-			LOG(LOG_INFO, _("Could not create cache directory, falling back to default cache directory: ") <<
-					defaultCacheDirectory);
-			cacheDirectory = defaultCacheDirectory;
-		}
+		LOG(LOG_INFO, _("Could not create cache directory, falling back to default cache directory: ") <<
+				defaultCacheDirectory);
+		cacheDirectory = defaultCacheDirectory;
 	}
-
-	/* If no audio backend was specified, use a default */
-	if(audioBackend == INVALID)
+	dataDirectory = cacheDirectory+G_DIR_SEPARATOR_S+"files";
+	if (g_mkdir_with_parents(dataDirectory.c_str(),S_IRUSR | S_IWUSR | S_IXUSR))
 	{
-#ifdef _WIN32
-		audioBackend = WINMM;
-#else
-		audioBackend = PULSEAUDIO;
-#endif
+		LOG(LOG_INFO, "Could not create data directory, storing user data may not be possible");
+		dataDirectory = "";
+	}
+	else
+	{
+		dataDirectory += G_DIR_SEPARATOR_S;
+		dataDirectory +="cXXXXXX";
+		char* tmpdir = new char[dataDirectory.length()+100];
+		strncpy(tmpdir,dataDirectory.c_str(),dataDirectory.length());
+		tmpdir[dataDirectory.length()] = 0x00;
+		tmpdir =g_mkdtemp(tmpdir);
+		if (!tmpdir)
+		{
+			LOG(LOG_INFO, "Could not create data directory, storing user data may not be possible");
+			dataDirectory = "";
+		}
+		else
+			dataDirectory = tmpdir;
 	}
-	//Set the audio backend name
-	audioBackendName = audioBackendNames[audioBackend];
 
 #ifdef _WIN32
 	std::string regGnashPath = readRegistryEntry("GnashPath");
@@ -162,16 +160,17 @@ Config::Config():
 			LOG(LOG_ERROR,"Could not get executable path!");
 		else
 		{
-			path gnash_exec_path = s;
-			if(is_regular_file(gnash_exec_path / "gtk-gnash.exe"))
+			
+			std::string gnash_exec_path = s;
+			if(g_file_test(std::string(gnash_exec_path + "gtk-gnash.exe").c_str(),G_FILE_TEST_EXISTS))
 			{
-				LOG(LOG_INFO,"Found gnash at " << (gnash_exec_path / "gtk-gnash.exe"));
-				gnashPath = (gnash_exec_path / "gtk-gnash.exe").string();
+				LOG(LOG_INFO,"Found gnash at " << (gnash_exec_path + "gtk-gnash.exe"));
+				gnashPath = std::string(gnash_exec_path + "gtk-gnash.exe").c_str();
 			}
-			else if(is_regular_file(gnash_exec_path / "sdl-gnash.exe"))
+			else if(g_file_test(std::string(gnash_exec_path + "sdl-gnash.exe").c_str(),G_FILE_TEST_EXISTS))
 			{
-				LOG(LOG_INFO,"Found gnash at " << (gnash_exec_path / "sdl-gnash.exe"));
-				gnashPath = (gnash_exec_path / "sdl-gnash.exe").string();
+				LOG(LOG_INFO,"Found gnash at " << gnash_exec_path + "sdl-gnash.exe");
+				gnashPath = std::string(gnash_exec_path + "sdl-gnash.exe").c_str();
 			}
 			else
 				LOG(LOG_ERROR, "Could not find gnash in " << gnash_exec_path);
@@ -202,15 +201,8 @@ void Config::handleEntry()
 	string group = parser->getGroup();
 	string key = parser->getKey();
 	string value = parser->getValue();
-	//Audio backend
-	if(group == "audio" && key == "backend" && value == audioBackendNames[PULSEAUDIO])
-		audioBackend = PULSEAUDIO;
-	else if(group == "audio" && key == "backend" && value == audioBackendNames[SDL])
-		audioBackend = SDL;
-	else if(group == "audio" && key == "backend" && value == audioBackendNames[WINMM])
-		 audioBackend = WINMM;
 	//Rendering
-	else if(group == "rendering" && key == "enabled")
+	if(group == "rendering" && key == "enabled")
 		renderingEnabled = atoi(value.c_str());
 	//Cache directory
 	else if(group == "cache" && key == "directory")
@@ -219,5 +211,5 @@ void Config::handleEntry()
 	else if(group == "cache" && key == "prefix")
 		cachePrefix = value;
 	else
-		throw ConfigException((string) _("Invalid entry encountered in configuration file") + ": '" + group + "/" + key + "'='" + value + "'");
+		LOG(LOG_ERROR,_("Invalid entry encountered in configuration file") << ": '" << group << "/" << key << "'='" << value << "'");
 }
