@@ -36,9 +36,9 @@ public:
 	T x,y;
 	Vector2Tmpl(T a=0, T b=0):x(a),y(b){}
 	/* conversion Vector2 -> Vector2f is implicit */
-	Vector2Tmpl(const Vector2Tmpl<int>& o) : x(o.x),y(o.y) {}
+	Vector2Tmpl(const Vector2Tmpl<int32_t>& o) : x(o.x),y(o.y) {}
 	/* conversion Vector2f -> Vector2 is explicit */
-	Vector2Tmpl<int> round() const { return Vector2Tmpl<int>(x,y); }
+	Vector2Tmpl<int32_t> round() const { return Vector2Tmpl<int32_t>(x,y); }
 	bool operator==(const Vector2Tmpl<T>& v)const{return v.x==x && v.y==y;}
 	bool operator!=(const Vector2Tmpl<T>& v)const{return v.x!=x || v.y!=y;}
 	bool operator<(const Vector2Tmpl<T>& v) const {return (y==v.y)?(x < v.x):(y < v.y);}
@@ -65,33 +65,50 @@ public:
 
 enum GEOM_TOKEN_TYPE { STRAIGHT=0, CURVE_QUADRATIC, MOVE, SET_FILL, SET_STROKE, CLEAR_FILL, CLEAR_STROKE, CURVE_CUBIC, FILL_KEEP_SOURCE, FILL_TRANSFORM_TEXTURE };
 
-class GeomToken :public RefCountable
+struct GeomToken
 {
-public:
-	FILLSTYLE  fillStyle;
-	LINESTYLE2 lineStyle;
-	MATRIX textureTransform;
-	GEOM_TOKEN_TYPE type;
-	Vector2 p1;
-	Vector2 p2;
-	Vector2 p3;
-	GeomToken(GEOM_TOKEN_TYPE _t):fillStyle(0xff),lineStyle(0xff),type(_t),p1(0,0),p2(0,0),p3(0,0){}
-	GeomToken(GEOM_TOKEN_TYPE _t, const Vector2& _p):fillStyle(0xff),lineStyle(0xff),type(_t),p1(_p),p2(0,0),p3(0,0){}
-	GeomToken(GEOM_TOKEN_TYPE _t, const Vector2& _p1, const Vector2& _p2):fillStyle(0xff),lineStyle(0xff),type(_t),p1(_p1),p2(_p2),p3(0,0){}
-	GeomToken(GEOM_TOKEN_TYPE _t, const Vector2& _p1, const Vector2& _p2, const Vector2& _p3):fillStyle(0xff),lineStyle(0xff),type(_t),
-			p1(_p1),p2(_p2),p3(_p3){}
-	GeomToken(GEOM_TOKEN_TYPE _t, const FILLSTYLE&  _f):fillStyle(_f),lineStyle(0xff),type(_t),p1(0,0),p2(0,0),p3(0,0){}
-	GeomToken(GEOM_TOKEN_TYPE _t, const LINESTYLE2& _s):fillStyle(0xff),lineStyle(_s),type(_t),p1(0,0),p2(0,0),p3(0,0){}
-	GeomToken(GEOM_TOKEN_TYPE _t, const MATRIX _m):fillStyle(0xff),lineStyle(0xff),textureTransform(_m),type(_t),p1(0,0),p2(0,0),p3(0,0){}
-	GeomToken(GEOM_TOKEN_TYPE _t, const MORPHLINESTYLE2& _s);
+	union
+	{
+		GEOM_TOKEN_TYPE type;
+		struct
+		{
+			int32_t x;
+			int32_t y;
+		} vec;
+		const FILLSTYLE*  fillStyle; // make sure the pointer is valid until rendering is done
+		const LINESTYLE2* lineStyle; // make sure the pointer is valid until rendering is done
+		const MORPHLINESTYLE2* morphlineStyle; // make sure the pointer is valid until rendering is done
+		number_t value;
+		uint64_t uval;// this is used to have direct access to the value as it is stored in a vector<uint64_t> for performance
+	};
+	GeomToken(GEOM_TOKEN_TYPE t):type(t) {}
+	GeomToken(uint64_t v, bool isvec)
+	{
+		if (isvec)
+		{
+			vec.x = (int32_t(v&0xffffffff));
+			vec.y = (int32_t(v>>32));
+		}
+		else
+			uval = v;
+	}
+	GeomToken(const FILLSTYLE& fs):fillStyle(&fs) {}
+	GeomToken(const LINESTYLE2& ls):lineStyle(&ls) {}
+	GeomToken(number_t val):value(val) {}
+	GeomToken(const MORPHLINESTYLE2& mls):morphlineStyle(&mls) {}
+	GeomToken(const Vector2& _vec)
+	{
+		vec.x=_vec.x;
+		vec.y=_vec.y;
+	}
 };
 
 struct tokensVector
 {
-	std::vector<_NR<GeomToken>, reporter_allocator<_NR<GeomToken>>> filltokens;
-	std::vector<_NR<GeomToken>, reporter_allocator<_NR<GeomToken>>> stroketokens;
-	tokensVector(reporter_allocator<_NR<GeomToken>> m):filltokens(m),stroketokens(m) {}
-	void clear() 
+	std::vector<uint64_t> filltokens;
+	std::vector<uint64_t> stroketokens;
+	tokensVector() {}
+	void clear()
 	{
 		filltokens.clear();
 		stroketokens.clear();
@@ -111,41 +128,34 @@ enum SHAPE_PATH_SEGMENT_TYPE { PATH_START=0, PATH_STRAIGHT, PATH_CURVE_QUADRATIC
 class ShapePathSegment {
 public:
 	SHAPE_PATH_SEGMENT_TYPE type;
-	unsigned int i;
-	ShapePathSegment(SHAPE_PATH_SEGMENT_TYPE _t, unsigned int _i):type(_t),i(_i){}
+	uint64_t i;
+	ShapePathSegment(SHAPE_PATH_SEGMENT_TYPE _t, uint64_t _i):type(_t),i(_i){}
 	bool operator==(const ShapePathSegment& v)const{return v.i == i;}
 };
 
 class ShapesBuilder
 {
 private:
-	std::map< Vector2, unsigned int > verticesMap;
-	// also add a vector of the vertices for performance
-	std::vector< Vector2> verticesVector;
-	std::map< unsigned int, std::vector< std::vector<ShapePathSegment> > > filledShapesMap;
-	std::map< unsigned int, std::vector< std::vector<ShapePathSegment> > > strokeShapesMap;
 	void joinOutlines();
 	static bool isOutlineEmpty(const std::vector<ShapePathSegment>& outline);
-	static void extendOutlineForColor(std::map< unsigned int, std::vector< std::vector<Vector2> > >& map);
-	unsigned int makeVertex(const Vector2& v);
-	const Vector2& getVertex(unsigned int index);
+	inline uint64_t makeVertex(const Vector2& v) const { return (uint64_t(v.y)<<32) | (uint64_t(v.x)&0xffffffff); }
 public:
-	void extendFilledOutlineForColor(unsigned int fillColor, const Vector2& v1, const Vector2& v2);
-	void extendFilledOutlineForColorCurve(unsigned int color, const Vector2& start, const Vector2& control, const Vector2& end);
-	void extendStrokeOutline(unsigned int stroke, const Vector2& v1, const Vector2& v2);
-	void extendStrokeOutlineCurve(unsigned int color, const Vector2& v1, const Vector2& v2, const Vector2& v3);
+	std::map< unsigned int, std::vector< std::vector<ShapePathSegment> > > filledShapesMap;
+	std::map< unsigned int, std::vector< std::vector<ShapePathSegment> > > strokeShapesMap;
+	std::vector<ShapePathSegment>* extendOutline(std::vector<std::vector<ShapePathSegment> >* outlines, const Vector2& v1, const Vector2& v2, std::vector<ShapePathSegment>* lastOutline);
+	std::vector<ShapePathSegment>* extendOutlineCurve(std::vector<std::vector<ShapePathSegment> >* outlines, const Vector2& start, const Vector2& control, const Vector2& end, std::vector<ShapePathSegment>* lastOutline);
 	/**
 		Generate a sequence of cachable tokens that defines the geomtries
 		@param styles This list is supposed to survive until as long as the returned tokens array
 		@param tokens A vector that will be filled with tokens
 	*/
-	void outputTokens(const std::list<FILLSTYLE>& styles, const std::list<LINESTYLE2> &linestyles, tokensVector& tokens);
+	void outputTokens(const std::list<FILLSTYLE>& styles, const std::list<LINESTYLE2>& linestyles, tokensVector& tokens);
 	void outputMorphTokens(const std::list<MORPHFILLSTYLE>& styles, const std::list<MORPHLINESTYLE2> &linestyles, tokensVector& tokens, uint16_t ratio);
 	void clear();
 };
 
 std::ostream& operator<<(std::ostream& s, const Vector2& p);
 
-};
+}
 
 #endif /* BACKENDS_GEOMETRY_H */
