@@ -72,7 +72,7 @@ class CachedSurface
 public:
 	CachedSurface():tex(nullptr),xOffset(0),yOffset(0),xOffsetTransformed(0),yOffsetTransformed(0),widthTransformed(0),heightTransformed(0),alpha(1.0),rotation(0.0),xscale(1.0),yscale(1.0)
 	  , redMultiplier(1.0), greenMultiplier(1.0), blueMultiplier(1.0), alphaMultiplier(1.0), redOffset(0.0), greenOffset(0.0), blueOffset(0.0), alphaOffset(0.0)
-	  ,isMask(false),hasMask(false),isChunkOwner(true){}
+	  ,isMask(false),hasMask(false),smoothing(true),isChunkOwner(true){}
 	~CachedSurface()
 	{
 		if (isChunkOwner && tex)
@@ -99,6 +99,7 @@ public:
 	float alphaOffset;
 	bool isMask;
 	bool hasMask;
+	bool smoothing;
 	bool isChunkOwner;
 };
 
@@ -112,7 +113,7 @@ public:
 	/*
 		Upload data to memory mapped to the graphics card (note: size is guaranteed to be enough
 	*/
-	virtual void upload(uint8_t* data, uint32_t w, uint32_t h) const=0;
+	virtual void upload(uint8_t* data, uint32_t w, uint32_t h)=0;
 	virtual const TextureChunk& getTexture()=0;
 	/*
 		Signal the completion of the upload to the texture
@@ -164,6 +165,7 @@ protected:
 	float alphaOffset;
 	bool isMask;
 	bool hasMask;
+	bool smoothing;
 public:
 	IDrawable(int32_t w, int32_t h, int32_t x, int32_t y,
 		int32_t rw, int32_t rh, int32_t rx, int32_t ry, float r,
@@ -171,19 +173,19 @@ public:
 		bool im, bool hm,
 		float a, const std::vector<MaskData>& m,
 		float _redMultiplier,float _greenMultiplier,float _blueMultiplier,float _alphaMultiplier,
-		float _redOffset,float _greenOffset,float _blueOffset,float _alphaOffset):
+		float _redOffset,float _greenOffset,float _blueOffset,float _alphaOffset, bool _smoothing):
 		masks(m),width(w),height(h),xOffset(x),yOffset(y),xOffsetTransformed(rx),yOffsetTransformed(ry),widthTransformed(rw),heightTransformed(rh),rotation(r),
 		alpha(a),xscale(xs),yscale(ys),
 		redMultiplier(_redMultiplier),greenMultiplier(_greenMultiplier),blueMultiplier(_blueMultiplier),alphaMultiplier(_alphaMultiplier),
 		redOffset(_redOffset),greenOffset(_greenOffset),blueOffset(_blueOffset),alphaOffset(_alphaOffset),
-		isMask(im),hasMask(hm) {}
+		isMask(im),hasMask(hm),smoothing(_smoothing) {}
 	virtual ~IDrawable();
 	/*
 	 * This method returns a raster buffer of the image
 	 * The various implementation are responsible for applying the
 	 * masks
 	 */
-	virtual uint8_t* getPixelBuffer(float scalex,float scaley)=0;
+	virtual uint8_t* getPixelBuffer(float scalex,float scaley, bool* isBufferOwner=nullptr)=0;
 	/*
 	 * This method creates a cairo path that can be used as a mask for
 	 * another object
@@ -203,6 +205,7 @@ public:
 	float getYScale() const { return yscale; }
 	bool getIsMask() const { return isMask; }
 	bool getHasMask() const { return hasMask; }
+	bool getSmoothing() const { return smoothing; }
 	float getRedMultiplier() const { return redMultiplier; }
 	float getGreenMultiplier() const { return greenMultiplier; }
 	float getBlueMultiplier() const { return blueMultiplier; }
@@ -234,14 +237,14 @@ public:
 	AsyncDrawJob(IDrawable* d, _R<DisplayObject> o);
 	~AsyncDrawJob();
 	//IThreadJob interface
-	void execute();
-	void threadAbort();
-	void jobFence();
+	void execute() override;
+	void threadAbort() override;
+	void jobFence() override;
 	//ITextureUploadable interface
-	void upload(uint8_t* data, uint32_t w, uint32_t h) const;
-	void sizeNeeded(uint32_t& w, uint32_t& h) const;
-	const TextureChunk& getTexture();
-	void uploadFence();
+	void upload(uint8_t* data, uint32_t w, uint32_t h) override;
+	void sizeNeeded(uint32_t& w, uint32_t& h) const override;
+	const TextureChunk& getTexture() override;
+	void uploadFence() override;
 	DisplayObject* getOwner() { return owner.getPtr(); }
 };
 
@@ -257,7 +260,6 @@ protected:
 	   Useful to adapt points defined in pixels and twips (1/20 of pixel)
 	*/
 	const float scaleFactor;
-	bool smoothing;
 	/**
 	  The whole transformation matrix that is applied to the rendered object
 	*/
@@ -279,7 +281,7 @@ public:
 				  , float _redOffset,float _greenOffset,float _blueOffset,float _alphaOffset
 				  , bool _smoothing,number_t _xstart,number_t _ystart);
 	//IDrawable interface
-	uint8_t* getPixelBuffer(float scalex, float scaley);
+	uint8_t* getPixelBuffer(float scalex, float scaley, bool* isBufferOwner=nullptr) override;
 	/*
 	 * Converts data (which is in RGB format) to the format internally used by cairo.
 	 */
@@ -305,8 +307,8 @@ private:
 	/*
 	 * This is run by CairoRenderer::execute()
 	 */
-	void executeDraw(cairo_t* cr, float scalex, float scaley);
-	void applyCairoMask(cairo_t* cr, int32_t offsetX, int32_t offsetY, float scalex, float scaley) const;
+	void executeDraw(cairo_t* cr, float scalex, float scaley) override;
+	void applyCairoMask(cairo_t* cr, int32_t offsetX, int32_t offsetY, float scalex, float scaley) const override;
 public:
 	/*
 	   CairoTokenRenderer constructor
@@ -427,6 +429,24 @@ public:
 	static std::vector<LineData> getLineData(const TextData& _textData);
 };
 
+class BitmapRenderer: public IDrawable
+{
+protected:
+	_NR<BitmapContainer> data;
+public:
+	BitmapRenderer(_NR<BitmapContainer> _data, int32_t _x, int32_t _y, int32_t _w, int32_t _h
+				  , int32_t _rx, int32_t _ry, int32_t _rw, int32_t _rh, float _r
+				  , float _xs, float _ys
+				  , bool _im, bool _hm
+				  , float _a, const std::vector<MaskData>& m
+				  , float _redMultiplier, float _greenMultiplier, float _blueMultiplier, float _alphaMultiplier
+				  , float _redOffset, float _greenOffset, float _blueOffset, float _alphaOffset
+				  , bool _smoothing);
+	//IDrawable interface
+	uint8_t* getPixelBuffer(float scalex, float scaley, bool* isBufferOwner=nullptr) override;
+	void applyCairoMask(cairo_t* cr, int32_t offsetX, int32_t offsetY, float scalex, float scaley) const override {}
+};
+
 class InvalidateQueue
 {
 public:
@@ -439,7 +459,23 @@ class SoftwareInvalidateQueue: public InvalidateQueue
 {
 public:
 	std::list<_R<DisplayObject>> queue;
-	void addToInvalidateQueue(_R<DisplayObject> d);
+	void addToInvalidateQueue(_R<DisplayObject> d) override;
+};
+
+class CharacterRenderer : public ITextureUploadable
+{
+	uint8_t* data;
+	uint32_t width;
+	uint32_t height;
+	TextureChunk chunk;
+public:
+	CharacterRenderer(uint8_t *d, uint32_t w, uint32_t h):data(d),width(w),height(h) {}
+	virtual ~CharacterRenderer() { delete[] data; }
+	//ITextureUploadable interface
+	void sizeNeeded(uint32_t& w, uint32_t& h) const override { w=width; h=height;}
+	void upload(uint8_t* data, uint32_t w, uint32_t h) override;
+	const TextureChunk& getTexture() override;
+	void uploadFence() override {}
 };
 
 }
