@@ -63,7 +63,7 @@ public:
 			float alpha, COLOR_MODE colorMode,float rotate, int32_t xtransformed, int32_t ytransformed, int32_t widthtransformed, int32_t heighttransformed, float xscale, float yscale,
 			float redMultiplier, float greenMultiplier, float blueMultiplier, float alphaMultiplier,
 			float redOffset, float greenOffset, float blueOffset, float alphaOffset,
-			bool isMask, bool hasMask)=0;
+			bool isMask, bool hasMask, float directMode, RGB directColor,bool smooth)=0;
 	/**
 	 * Get the right CachedSurface from an object
 	 */
@@ -90,6 +90,8 @@ protected:
 	int maskUniform;
 	int colortransMultiplyUniform;
 	int colortransAddUniform;
+	int directUniform;
+	int directColorUniform;
 	uint32_t maskframebuffer;
 	uint32_t maskTextureID;
 
@@ -114,7 +116,7 @@ public:
 	 * Uploads the current matrix as the specified type.
 	 */
 	void setMatrixUniform(LSGL_MATRIX m) const;
-	GLRenderContext() : RenderContext(GL),engineData(NULL), largeTextureSize(0)
+	GLRenderContext() : RenderContext(GL),engineData(nullptr), largeTextureSize(0)
 	{
 	}
 	void SetEngineData(EngineData* data) { engineData = data;}
@@ -124,13 +126,13 @@ public:
 			float alpha, COLOR_MODE colorMode, float rotate, int32_t xtransformed, int32_t ytransformed, int32_t widthtransformed, int32_t heighttransformed, float xscale, float yscale,
 			float redMultiplier, float greenMultiplier, float blueMultiplier, float alphaMultiplier,
 			float redOffset, float greenOffset, float blueOffset, float alphaOffset,
-			bool isMask, bool hasMask) override;
+			bool isMask, bool hasMask, float directMode, RGB directColor,bool smooth) override;
 	/**
 	 * Get the right CachedSurface from an object
 	 * In the OpenGL case we just get the CachedSurface inside the object itself
 	 */
-	const CachedSurface& getCachedSurface(const DisplayObject* obj) const;
-	void setProperties(AS_BLENDMODE blendmode);
+	const CachedSurface& getCachedSurface(const DisplayObject* obj) const override;
+	void setProperties(AS_BLENDMODE blendmode) override;
 
 	/* Utility */
 	bool handleGLErrors() const;
@@ -153,19 +155,19 @@ public:
 			float alpha, COLOR_MODE colorMode, float rotate, int32_t xtransformed, int32_t ytransformed, int32_t widthtransformed, int32_t heighttransformed, float xscale, float yscale,
 			float redMultiplier, float greenMultiplier, float blueMultiplier, float alphaMultiplier,
 			float redOffset, float greenOffset, float blueOffset, float alphaOffset,
-			bool isMask, bool hasMask) override;
+			bool isMask, bool hasMask, float directMode, RGB directColor,bool smooth) override;
 	/**
 	 * Get the right CachedSurface from an object
 	 * In the Cairo case we get the right CachedSurface out of the map
 	 */
-	const CachedSurface& getCachedSurface(const DisplayObject* obj) const;
-	void setProperties(AS_BLENDMODE blendmode);
+	const CachedSurface& getCachedSurface(const DisplayObject* obj) const override;
+	void setProperties(AS_BLENDMODE blendmode) override;
 
 	/**
 	 * The CairoRenderContext acquires the ownership of the buffer
 	 * it will be freed on destruction
 	 */
-	CachedSurface& allocateCustomSurface(const DisplayObject* d, uint8_t* texBuf);
+	CachedSurface& allocateCustomSurface(const DisplayObject* d, uint8_t* texBuf, bool isBufferOwner);
 	/**
 	 * Do a fast non filtered, non scaled blit of ARGB data
 	 */
