@@ -1282,7 +1282,8 @@ bool SocketPolicyFile::retrievePolicyFile(vector<unsigned char>& outData)
 	tiny_string hostname = url.getHostname();
 	uint16_t port = url.getPort();
 	SocketIO sock;
-	if (!sock.connect(hostname, port))
+	// according to http://www.lightsphere.com/dev/articles/flash_socket_policy.html adobe has a 3 second timeout when retrieving the socket policy file
+	if (!sock.connect(hostname, port,3))
 	{
 		if (isMaster())
 		{
@@ -1313,7 +1314,7 @@ bool SocketPolicyFile::retrievePolicyFile(vector<unsigned char>& outData)
 		if (nbytes > 0)
 			outData.insert(outData.end(), buf, buf + nbytes);
 	}
-	while (nbytes > 0);
+	while (nbytes == 4096);
 
 	if (nbytes < 0 && outData.size() == 0)
 	{
@@ -1394,7 +1395,7 @@ PolicyAllowAccessFrom::PolicyAllowAccessFrom(PolicyFile* _file, const string _do
 	if(!secureSpecified)
 	{
 		if(file->getType() == PolicyFile::URL &&
-				dynamic_cast<URLPolicyFile*>(file)->getSubtype() == URLPolicyFile::HTTPS)
+				static_cast<URLPolicyFile*>(file)->getSubtype() == URLPolicyFile::HTTPS)
 			secure = true;
 		if(file->getType() == PolicyFile::SOCKET)
 			secure = false;
@@ -1485,7 +1486,7 @@ bool PolicyAllowAccessFrom::allowsAccessFrom(const URLInfo& url, uint16_t toPort
 	if (bCheckHttps)
 	{
 		if(file->getType() == PolicyFile::URL && 
-				dynamic_cast<URLPolicyFile*>(file)->getSubtype() == URLPolicyFile::HTTPS && 
+				static_cast<URLPolicyFile*>(file)->getSubtype() == URLPolicyFile::HTTPS && 
 				secure && url.getProtocol() != "https")
 			return false;
 		if(file->getType() == PolicyFile::SOCKET && 
