@@ -88,7 +88,6 @@ public:
 	virtual int getId() const=0;
 	virtual ASObject* instance(Class_base* c=nullptr) { return nullptr; }
 	virtual MATRIX MapToBounds(const MATRIX& mat) { return mat; }
-	virtual MATRIX MapToBoundsForButton(const MATRIX& mat) { return MapToBounds(mat); }
 	virtual void resizeCompleted() {}
 };
 
@@ -153,9 +152,8 @@ protected:
 public:
 	DefineShapeTag(RECORDHEADER h,std::istream& in, RootMovieClip* root);
 	~DefineShapeTag();
-	virtual int getId() const{ return ShapeId; }
-	ASObject* instance(Class_base* c=nullptr);
-	MATRIX MapToBoundsForButton(const MATRIX& mat) override;
+	int getId() const override { return ShapeId; }
+	ASObject* instance(Class_base* c=nullptr) override;
 	void resizeCompleted() override;
 };
 
@@ -173,7 +171,7 @@ protected:
 	DefineShape3Tag(RECORDHEADER h, int v, RootMovieClip* root):DefineShape2Tag(h,v,root){}
 public:
 	DefineShape3Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	virtual int getId() const { return ShapeId; }
+	int getId() const override { return ShapeId; }
 };
 
 class DefineShape4Tag: public DefineShape3Tag
@@ -185,7 +183,7 @@ private:
 	UB UsesScalingStrokes;
 public:
 	DefineShape4Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	virtual int getId() const { return ShapeId; }
+	int getId() const override { return ShapeId; }
 };
 
 class DefineMorphShapeTag: public DictionaryTag
@@ -199,11 +197,13 @@ protected:
 	MORPHLINESTYLEARRAY MorphLineStyles;
 	SHAPE StartEdges;
 	SHAPE EndEdges;
+	std::map<uint32_t,tokensVector> tokensmap;
 	DefineMorphShapeTag(RECORDHEADER h, RootMovieClip* root, int version):DictionaryTag(h,root),MorphLineStyles(version){}
 public:
 	DefineMorphShapeTag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	int getId() const { return CharacterId; }
-	virtual ASObject* instance(Class_base* c=NULL);
+	int getId() const override { return CharacterId; }
+	ASObject* instance(Class_base* c=nullptr) override;
+	void getTokensForRatio(tokensVector& tokens, uint32_t ratio);
 };
 
 class DefineMorphShape2Tag: public DefineMorphShapeTag
@@ -219,6 +219,7 @@ public:
 
 class DefineEditTextTag: public DictionaryTag
 {
+friend class TextField;
 private:
 	UI16_SWF CharacterID;
 	RECT Bounds;
@@ -273,7 +274,7 @@ private:
 	int realSampleRate;
 public:
 	DefineSoundTag(RECORDHEADER h, std::istream& s, RootMovieClip* root);
-	virtual int getId() const { return SoundId; }
+	int getId() const override { return SoundId; }
 	ASObject* instance(Class_base* c=nullptr) override;
 	LS_AUDIO_CODEC getAudioCodec() const;
 	number_t getDurationInMS() const;
@@ -398,7 +399,7 @@ private:
 
 public:
 	PlaceObject3Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	void setProperties(DisplayObject* obj, DisplayObjectContainer* parent) const;
+	void setProperties(DisplayObject* obj, DisplayObjectContainer* parent) const override;
 };
 
 class FrameLabelTag: public Tag
@@ -415,7 +416,7 @@ private:
 	RGB BackgroundColor;
 public:
 	SetBackgroundColorTag(RECORDHEADER h, std::istream& in);
-	void execute(RootMovieClip* root) const;
+	void execute(RootMovieClip* root) const override;
 };
 
 class DefineButtonTag: public DictionaryTag
@@ -428,8 +429,8 @@ private:
 public:
 	std::vector<BUTTONCONDACTION> condactions;
 	DefineButtonTag(RECORDHEADER h, std::istream& in, int version, RootMovieClip* root, AdditionalDataTag* datatag);
-	virtual int getId() const { return ButtonId; }
-	ASObject* instance(Class_base* c=NULL);
+	int getId() const override { return ButtonId; }
+	ASObject* instance(Class_base* c=nullptr) override;
 };
 
 class KERNINGRECORD
@@ -446,8 +447,8 @@ private:
 public:
 	DefineBinaryDataTag(RECORDHEADER h,std::istream& s,RootMovieClip* root);
 	~DefineBinaryDataTag() { delete[] bytes; }
-	virtual int getId() const {return Tag;}
-	ASObject* instance(Class_base* c=NULL);
+	int getId() const override {return Tag;}
+	ASObject* instance(Class_base* c=nullptr) override;
 };
 
 class FontTag: public DictionaryTag
@@ -463,32 +464,46 @@ protected:
 	bool FontFlagsANSI;
 	bool FontFlagsItalic;
 	bool FontFlagsBold;
+	virtual number_t getRenderCharStartYPos() const =0;
+	std::list<FILLSTYLE> fillStyles;
 public:
 	/* Multiply the coordinates of the SHAPEs by this
 	 * value to get a resolution of 1024*20th pixel
 	 * DefineFont3Tag sets 1 here, the rest set 20
 	 */
 	const int scaling;
-	FontTag(RECORDHEADER h, int _scaling,RootMovieClip* root):DictionaryTag(h,root), scaling(_scaling) {}
-	const std::vector<SHAPE>& getGlyphShapes() const
+	FontTag(RECORDHEADER h, int _scaling,RootMovieClip* root);
+	std::vector<SHAPE>& getGlyphShapes()
 	{
 		return GlyphShapeTable;
 	}
-	int getId() const { return FontID; }
-	ASObject* instance(Class_base* c=NULL);
+	int getId() const override { return FontID; }
+	ASObject* instance(Class_base* c=nullptr) override;
 	const tiny_string getFontname() const { return fontname;}
-	virtual void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, RGB textColor, uint32_t leading,uint32_t startpos) const=0;
+	virtual void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, FILLSTYLE& fillstyleColor, uint32_t leading,uint32_t startpos)=0;
+	virtual number_t getRenderCharAdvance(uint32_t index) const =0;
+	virtual void getTextBounds(const tiny_string& text, int fontpixelsize, number_t& width, number_t& height)=0;
+	const TextureChunk *getCharTexture(const CharIterator& chrIt, int fontpixelsize, uint32_t &codetableindex);
 	bool hasGlyphs(const tiny_string text) const;
+	virtual int32_t getLeading() const =0;
+	virtual int32_t getAscent() const =0;
+	virtual int32_t getDescent() const =0;
 };
 
 class DefineFontTag: public FontTag
 {
-	friend class DefineTextTag; 
+	friend class DefineTextTag;
 protected:
 	std::vector<uint16_t> OffsetTable;
+	number_t getRenderCharStartYPos() const override;
 public:
 	DefineFontTag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, RGB textColor, uint32_t leading,uint32_t startpos) const;
+	number_t getRenderCharAdvance(uint32_t index) const override;
+	void getTextBounds(const tiny_string& text, int fontpixelsize, number_t& width, number_t& height) override;
+	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, FILLSTYLE& fillstyleColor, uint32_t leading,uint32_t startpos) override;
+	int32_t getLeading() const override { return 1024; }
+	int32_t getAscent() const override { return 1024; }
+	int32_t getDescent() const override { return 1024; }
 };
 
 class DefineFontInfoTag: public Tag
@@ -515,10 +530,16 @@ private:
 	std::vector < RECT > FontBoundsTable;
 	UI16_SWF KerningCount;
 	std::vector <KERNINGRECORD> FontKerningTable;
-
+protected:
+	number_t getRenderCharStartYPos() const override;
 public:
 	DefineFont2Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, RGB textColor, uint32_t leading,uint32_t startpos) const;
+	number_t getRenderCharAdvance(uint32_t index) const override;
+	void getTextBounds(const tiny_string& text, int fontpixelsize, number_t& width, number_t& height) override;
+	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, FILLSTYLE& fillstyleColor, uint32_t leading,uint32_t startpos) override;
+	int32_t getLeading() const override { return FontLeading; }
+	int32_t getAscent() const override { return FontAscent; }
+	int32_t getDescent() const override { return FontDescent; }
 };
 
 class DefineFont3Tag: public FontTag
@@ -538,10 +559,16 @@ private:
 	std::vector < RECT > FontBoundsTable;
 	UI16_SWF KerningCount;
 	std::vector <KERNINGRECORD> FontKerningTable;
-
+protected:
+	number_t getRenderCharStartYPos() const override;
 public:
 	DefineFont3Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, RGB textColor, uint32_t leading,uint32_t startpos) const;
+	number_t getRenderCharAdvance(uint32_t index) const override;
+	void getTextBounds(const tiny_string& text, int fontpixelsize, number_t& width, number_t& height) override;
+	void fillTextTokens(tokensVector &tokens, const tiny_string text, int fontpixelsize, FILLSTYLE& fillstyleColor, uint32_t leading,uint32_t startpos) override;
+	int32_t getLeading() const override { return FontLeading/20; }
+	int32_t getAscent() const override { return FontAscent/20; }
+	int32_t getDescent() const override { return FontDescent/20; }
 };
 
 class DefineFont4Tag : public DictionaryTag
@@ -554,8 +581,8 @@ private:
 	STRING FontName;
 public:
 	DefineFont4Tag(RECORDHEADER h, std::istream& in,RootMovieClip* root);
-	virtual int getId() const { return FontID; }
-	ASObject* instance(Class_base* c=nullptr);
+	int getId() const override { return FontID; }
+	ASObject* instance(Class_base* c=nullptr) override;
 };
 
 class DefineTextTag: public DictionaryTag
@@ -569,12 +596,13 @@ private:
 	UI8 AdvanceBits;
 	std::vector < TEXTRECORD > TextRecords;
 	mutable tokensVector tokens;
-	void computeCached() const;
+	std::list<FILLSTYLE> fillStyles;
+	void computeCached();
 public:
 	int version;
 	DefineTextTag(RECORDHEADER h, std::istream& in,RootMovieClip* root,int v=1);
-	int getId() const { return CharacterId; }
-	ASObject* instance(Class_base* c=nullptr);
+	int getId() const override { return CharacterId; }
+	ASObject* instance(Class_base* c=nullptr) override;
 };
 
 class DefineText2Tag: public DefineTextTag
@@ -593,8 +621,8 @@ public:
 	SoundStreamHeadTag* soundheadtag;
 	DefineSpriteTag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
 	~DefineSpriteTag();
-	virtual int getId() const { return SpriteID; }
-	virtual ASObject* instance(Class_base* c=nullptr);
+	int getId() const override { return SpriteID; }
+	ASObject* instance(Class_base* c=nullptr) override;
 	void setSoundStartFrame(uint32_t frame) 
 	{
 		if (soundstartframe == UINT32_MAX)
@@ -606,7 +634,7 @@ class ProtectTag: public ControlTag
 {
 public:
 	ProtectTag(RECORDHEADER h, std::istream& in);
-	void execute(RootMovieClip* root) const{}
+	void execute(RootMovieClip* root) const override {}
 };
 
 class BitmapContainer;
@@ -618,8 +646,8 @@ protected:
     void loadBitmap(uint8_t* inData, int datasize, const uint8_t *tablesData=nullptr, int tablesLen=0);
 public:
 	BitmapTag(RECORDHEADER h,RootMovieClip* root);
-	ASObject* instance(Class_base* c=nullptr);
-        _R<BitmapContainer> getBitmap() const;
+	ASObject* instance(Class_base* c=nullptr) override;
+	_R<BitmapContainer> getBitmap() const;
 };
 
 class JPEGTablesTag: public Tag
@@ -644,7 +672,7 @@ private:
 	//ZlibBitmapData;
 public:
 	DefineBitsLosslessTag(RECORDHEADER h, std::istream& in, int version, RootMovieClip* root);
-	int getId() const{ return CharacterId; }
+	int getId() const override { return CharacterId; }
 };
 
 class DefineBitsTag: public BitmapTag
@@ -653,7 +681,7 @@ private:
 	UI16_SWF CharacterId;
 public:
 	DefineBitsTag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	int getId() const{ return CharacterId; }
+	int getId() const override { return CharacterId; }
 };
 
 class DefineBitsJPEG2Tag: public BitmapTag
@@ -662,7 +690,7 @@ private:
 	UI16_SWF CharacterId;
 public:
 	DefineBitsJPEG2Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
-	int getId() const{ return CharacterId; }
+	int getId() const override { return CharacterId; }
 };
 
 class DefineBitsJPEG3Tag: public BitmapTag
@@ -673,7 +701,7 @@ private:
 public:
 	DefineBitsJPEG3Tag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
 	~DefineBitsJPEG3Tag();
-	int getId() const{ return CharacterId; }
+	int getId() const override { return CharacterId; }
 };
 
 class DefineScalingGridTag: public Tag
@@ -725,7 +753,7 @@ class DefineFontAlignZonesTag: public Tag
 public:
 	DefineFontAlignZonesTag(RECORDHEADER h, std::istream& in);
 };
-
+class VideoFrameTag;
 class DefineVideoStreamTag: public DictionaryTag
 {
 friend class Video;
@@ -738,10 +766,14 @@ private:
 	UB VideoFlagsDeblocking;
 	UB VideoFlagsSmoothing;
 	UI8 VideoCodecID;
+	vector<VideoFrameTag*> frames;
 public:
 	DefineVideoStreamTag(RECORDHEADER h, std::istream& in, RootMovieClip* root);
+	~DefineVideoStreamTag();
 	int getId() const override { return CharacterID; }
 	ASObject* instance(Class_base* c=nullptr) override;
+	void setFrameData(VideoFrameTag* tag);
+	VideoFrameTag* getFrame(uint32_t frame) const { return frames[frame]; }
 };
 class VideoFrameTag: public DisplayListTag
 {
@@ -752,7 +784,11 @@ private:
 	uint32_t numbytes;
 public:
 	VideoFrameTag(RECORDHEADER h, std::istream& in);
+	~VideoFrameTag();
 	void execute(DisplayObjectContainer* parent,bool inskipping) override;
+	uint8_t* getData() { return framedata; }
+	uint32_t getNumBytes() { return numbytes+AV_INPUT_BUFFER_PADDING_SIZE; }
+	uint32_t getFrameNumber() { return FrameNum; }
 };
 class MetadataTag: public Tag
 {
