@@ -277,8 +277,14 @@ void EngineData::initGLEW()
 	GLenum err = glewInit();
 	if (GLEW_OK != err)
 	{
-		LOG(LOG_ERROR,_("Cannot initialize GLEW: cause ") << glewGetErrorString(err));
-		throw RunTimeException("Rendering: Cannot initialize GLEW!");
+#ifdef GLEW_ERROR_NO_GLX_DISPLAY
+		char* videodriver = getenv("SDL_VIDEODRIVER"); // ignore GLEW_ERROR_NO_GLX_DISPLAY when running on wayland
+		if (!videodriver || strcmp(videodriver,"wayland")!= 0 || err != GLEW_ERROR_NO_GLX_DISPLAY)
+#endif
+		{
+			LOG(LOG_ERROR,_("Cannot initialize GLEW: cause ") << glewGetErrorString(err));
+			throw RunTimeException("Rendering: Cannot initialize GLEW!");
+		}
 	}
 
 	if(!GLEW_VERSION_2_0)
@@ -1343,7 +1349,11 @@ void EngineData::audio_ManagerCloseMixer()
 
 bool EngineData::audio_ManagerOpenMixer()
 {
-	return Mix_OpenAudio (audio_getSampleRate(), AUDIO_S16, 2, LIGHTSPARK_AUDIO_BUFFERSIZE) >= 0;
+#if __BYTE_ORDER == __BIG_ENDIAN
+	return Mix_OpenAudio (audio_getSampleRate(), AUDIO_S16MSB, 2, LIGHTSPARK_AUDIO_BUFFERSIZE) >= 0;
+#else
+	return Mix_OpenAudio (audio_getSampleRate(), AUDIO_S16LSB, 2, LIGHTSPARK_AUDIO_BUFFERSIZE) >= 0;
+#endif
 }
 
 void EngineData::audio_ManagerDeinit()
@@ -1374,12 +1384,14 @@ externalFontRenderer::externalFontRenderer(const TextData &_textData, EngineData
 										   bool smoothing)
 	: IDrawable(w, h, x, y,rw,rh,rx,ry,r,xs,ys,im,hm, a, m,
 				_redMultiplier,_greenMultiplier,_blueMultiplier,_alphaMultiplier,
-				_redOffset,_greenOffset,_blueOffset,_alphaOffset),m_engine(engine)
+				_redOffset,_greenOffset,_blueOffset,_alphaOffset,smoothing),m_engine(engine)
 {
 	externalressource = engine->setupFontRenderer(_textData,a,smoothing);
 }
 
-uint8_t *externalFontRenderer::getPixelBuffer(float scalex, float scaley)
+uint8_t *externalFontRenderer::getPixelBuffer(float scalex, float scaley, bool *isBufferOwner)
 {
+	if (isBufferOwner)
+		*isBufferOwner = true;
 	return m_engine->getFontPixelBuffer(externalressource,this->width,this->height);
 }
