@@ -134,7 +134,7 @@ EARLY_BIND_STATUS ABCVm::earlyBindForScopeStack(ostream& out, const SyntheticFun
 		const std::vector<InferenceData>& scopeStack, const multiname* name, InferenceData& inferredData)
 {
 	//We try to find out the position on the scope stack where the name is found
-	uint32_t totalScopeStackLen = f->func_scope.size() + scopeStack.size();
+	uint32_t totalScopeStackLen = f->func_scope->scope.size() + scopeStack.size();
 
 	bool found=false;
 	//Traverse in reverse order the scope stack. Stops at the first any/unknown type.
@@ -170,7 +170,7 @@ EARLY_BIND_STATUS ABCVm::earlyBindForScopeStack(ostream& out, const SyntheticFun
 		//TODO: We can also push directly those objects since they are fixed
 		std::cerr << "End of local stack" << std::endl;
 		//Now look in the objects that are stored in the function save scope stack
-		for(auto it=f->func_scope.rbegin();it!=f->func_scope.rend();++it)
+		for(auto it=f->func_scope->scope.rbegin();it!=f->func_scope->scope.rend();++it)
 		{
 			totalScopeStackLen--;
 			if(it->considerDynamic)
@@ -179,11 +179,11 @@ EARLY_BIND_STATUS ABCVm::earlyBindForScopeStack(ostream& out, const SyntheticFun
 				return CANNOT_BIND;
 			}
 
-			const variable* var=it->object->findVariableByMultiname(*name, ASObject::XML_STRICT, it->object->getClass());
+			const variable* var=asAtomHandler::toObject(it->object,f->getSystemState())->findVariableByMultiname(*name, asAtomHandler::toObject(it->object,f->getSystemState())->getClass());
 			if(var)
 			{
 				found=true;
-				inferredData.obj=it->object.getPtr();
+				inferredData.obj=asAtomHandler::toObject(it->object,f->getSystemState());
 				break;
 			}
 		}
@@ -239,13 +239,14 @@ InferenceData ABCVm::earlyBindGetLex(ostream& out, const SyntheticFunction* f, c
 	ASObject* target;
 	//Now we should serach in the applicationDomain. The system domain is the first one searched. We can safely
 	//early bind for it, but not for custom domains, since we may change the expected order of evaluation
-	ASObject* o=getSys()->systemDomain->getVariableAndTargetByMultiname(*name, target);
-	if(o)
+	asAtom o=asAtomHandler::invalidAtom;
+	f->getSystemState()->systemDomain->getVariableAndTargetByMultiname(o,*name, target);
+	if(asAtomHandler::isValid(o))
 	{
 		//Output a special opcode
 		out << (uint8_t)PUSH_EARLY;
-		writePtr(out, o);
-		ret.obj=o;
+		writePtr(out, asAtomHandler::toObject(o,f->getSystemState()));
+		ret.obj=asAtomHandler::getObject(o);
 		return ret;
 	}
 	//About custom domains. We can't resolve the object now. But we can output a special getLex opcode that will
@@ -334,6 +335,8 @@ void ABCVm::writeBranchAddress(std::map<uint32_t,BasicBlock>& basicBlocks, int h
 void ABCVm::optimizeFunction(SyntheticFunction* function)
 {
 	method_info* mi=function->mi;
+	SystemState* sys = function->getSystemState();
+	
 	ActivationType activationType(mi);
 
 	istringstream code(mi->body->code);
@@ -353,7 +356,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 	//Create a map of addresses to fixups to rewrite the exception data: from, to and target
 	for(uint32_t i=0;i<mi->body->exceptions.size();i++)
 	{
-		exception_info& ei=mi->body->exceptions[i];
+		exception_info_abc& ei=mi->body->exceptions[i];
 		//Also create a new basic block at the exception target address,
 		//otherwise that code might be considered unreachable and lost
 		if(basicBlocks.find(ei.target)==basicBlocks.end())
@@ -698,10 +701,18 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				out << (uint8_t)opcode;
 				break;
 			}
+			case 0x1f:
+			{
+				//hasnext
+				curBlock->popStack(2);
+				curBlock->pushStack(Class<Integer>::getClass(sys));
+				out << (uint8_t)opcode;
+				break;
+			}
 			case 0x20:
 			{
 				//pushnull
-				ASObject* ret=getSys()->getNullRef();
+				ASObject* ret=sys->getNullRef();
 				//We don't really need a reference to it
 				ret->decRef();
 				curBlock->pushStack(InferenceData(ret));
@@ -730,7 +741,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				code.read((char*)&t,1);
 				out << (uint8_t)opcode;
 				out << t;
-				curBlock->pushStack(Class<Integer>::getClass());
+				curBlock->pushStack(Class<Integer>::getClass(sys));
 				break;
 			}
 			case 0x25:
@@ -742,21 +753,21 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				code >> t;
 				out << (uint8_t)opcode;
 				writeInt32(out, t);
-				curBlock->pushStack(Class<Integer>::getClass());
+				curBlock->pushStack(Class<Integer>::getClass(sys));
 				break;
 			}
 			case 0x26:
 			{
 				//pushtrue
 				out << (uint8_t)opcode;
-				curBlock->pushStack(Class<Boolean>::getClass());
+				curBlock->pushStack(Class<Boolean>::getClass(sys));
 				break;
 			}
 			case 0x27:
 			{
 				//pushfalse
 				out << (uint8_t)opcode;
-				curBlock->pushStack(Class<Boolean>::getClass());
+				curBlock->pushStack(Class<Boolean>::getClass(sys));
 				break;
 			}
 			case 0x28:
@@ -811,7 +822,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				s32 val=mi->context->constant_pool.integer[t];
 				out << (uint8_t)opcode;
 				writeInt32(out, val);
-				curBlock->pushStack(Class<Integer>::getClass());
+				curBlock->pushStack(Class<Integer>::getClass(sys));
 				break;
 			}
 			case 0x2e:
@@ -822,7 +833,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				u32 val=mi->context->constant_pool.uinteger[t];
 				out << (uint8_t)opcode;
 				writeInt32(out, val);
-				curBlock->pushStack(Class<Integer>::getClass());
+				curBlock->pushStack(Class<Integer>::getClass(sys));
 				break;
 			}
 			case 0x2f:
@@ -833,7 +844,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				double val=mi->context->constant_pool.doubles[t];
 				out << (uint8_t)opcode;
 				writeDouble(out, val);
-				curBlock->pushStack(Class<Number>::getClass());
+				curBlock->pushStack(Class<Number>::getClass(sys));
 				break;
 			}
 			case 0x30:
@@ -864,7 +875,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				out << (uint8_t)opcode;
 				writeInt32(out,t);
 				writeInt32(out,t2);
-				curBlock->pushStack(Class<Boolean>::getClass());
+				curBlock->pushStack(Class<Boolean>::getClass(sys));
 				break;
 			}
 			//Alchemy opcodes
@@ -877,7 +888,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				//li32
 				out << (uint8_t)opcode;
 				curBlock->popStack(1);
-				curBlock->pushStack(Class<Integer>::getClass());
+				curBlock->pushStack(Class<Integer>::getClass(sys));
 				break;
 			}
 			case 0x3a:
@@ -923,6 +934,19 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				curBlock->pushStack(Type::anyType);
 				break;
 			}
+			case 0x43:
+			{
+				//callmethod
+				u30 t,t2;
+				code >> t;
+				code >> t2;
+				out << (uint8_t)opcode;
+				writeInt32(out,t);
+				writeInt32(out,t2);
+				curBlock->popStack(t2+1);
+				curBlock->pushStack(Type::anyType);
+				break;
+			}
 			case 0x4a:
 			{
 				//constructprop
@@ -965,9 +989,9 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 						if(objType)
 						{
 							const variable* var=objType->findBorrowedGettable(*name);
-							if(var && var->var && var->var->getObjectType()==T_FUNCTION)
+							if(var && asAtomHandler::isFunction(var->var))
 							{
-								SyntheticFunction* calledFunc=dynamic_cast<SyntheticFunction*>(var->var);
+								SyntheticFunction* calledFunc=dynamic_cast<SyntheticFunction*>(asAtomHandler::getObject(var->var));
 								if(calledFunc)
 									inferredData.type=calledFunc->mi->returnType;
 							}
@@ -1256,7 +1280,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 
 				int numRT=mi->context->getMultinameRTData(t);
 				curBlock->popStack(numRT+1);
-				curBlock->pushStack(Class<Boolean>::getClass());
+				curBlock->pushStack(Class<Boolean>::getClass(sys));
 				break;
 			}
 			case 0x6c:
@@ -1320,7 +1344,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				out << (uint8_t)opcode;
 
 				curBlock->popStack(1);
-				curBlock->pushStack(Class<ASString>::getClass());
+				curBlock->pushStack(Class<ASString>::getClass(sys));
 				break;
 			}
 			case 0x73:
@@ -1329,7 +1353,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				out << (uint8_t)opcode;
 
 				curBlock->popStack(1);
-				curBlock->pushStack(Class<Integer>::getClass());
+				curBlock->pushStack(Class<Integer>::getClass(sys));
 				break;
 			}
 			case 0x74:
@@ -1338,7 +1362,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				out << (uint8_t)opcode;
 
 				curBlock->popStack(1);
-				curBlock->pushStack(Class<UInteger>::getClass());
+				curBlock->pushStack(Class<UInteger>::getClass(sys));
 				break;
 			}
 			case 0x75:
@@ -1347,7 +1371,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				out << (uint8_t)opcode;
 
 				curBlock->popStack(1);
-				curBlock->pushStack(Class<Number>::getClass());
+				curBlock->pushStack(Class<Number>::getClass(sys));
 				break;
 			}
 			case 0x76:
@@ -1356,7 +1380,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				out << (uint8_t)opcode;
 
 				curBlock->popStack(1);
-				curBlock->pushStack(Class<Boolean>::getClass());
+				curBlock->pushStack(Class<Boolean>::getClass(sys));
 				break;
 			}
 			case 0x78:
@@ -1449,7 +1473,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				//decrement
 				out << (uint8_t)opcode;
 				curBlock->popStack(1);
-				curBlock->pushStack(Class<Number>::getClass());
+				curBlock->pushStack(Class<Number>::getClass(sys));
 				break;
 			}
 			case 0x92:
@@ -1468,7 +1492,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				//not
 				out << (uint8_t)opcode;
 				curBlock->popStack(1);
-				curBlock->pushStack(Class<Boolean>::getClass());
+				curBlock->pushStack(Class<Boolean>::getClass(sys));
 				break;
 			}
 			case 0x97:
@@ -1482,7 +1506,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				//negate_i
 				out << (uint8_t)opcode;
 				curBlock->popStack(1);
-				curBlock->pushStack(Class<Integer>::getClass());
+				curBlock->pushStack(Class<Integer>::getClass(sys));
 				break;
 			}
 			case 0xa0:
@@ -1505,7 +1529,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				//modulo
 				out << (uint8_t)opcode;
 				curBlock->popStack(2);
-				curBlock->pushStack(Class<Number>::getClass());
+				curBlock->pushStack(Class<Number>::getClass(sys));
 				break;
 			}
 			case 0xa5:
@@ -1529,7 +1553,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				//multiply_i
 				out << (uint8_t)opcode;
 				curBlock->popStack(2);
-				curBlock->pushStack(Class<Integer>::getClass());
+				curBlock->pushStack(Class<Integer>::getClass(sys));
 				break;
 			}
 			case 0xab:
@@ -1553,7 +1577,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				//in
 				out << (uint8_t)opcode;
 				curBlock->popStack(2);
-				curBlock->pushStack(Class<Boolean>::getClass());
+				curBlock->pushStack(Class<Boolean>::getClass(sys));
 				break;
 			}
 			case 0xb2:
@@ -1568,7 +1592,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 				if(numRT)
 					throw ParseException("Bad code in optimizer");
 				curBlock->popStack(1);
-				curBlock->pushStack(Class<Boolean>::getClass());
+				curBlock->pushStack(Class<Boolean>::getClass(sys));
 				break;
 			}
 			case 0xc2:
@@ -1649,7 +1673,7 @@ void ABCVm::optimizeFunction(SyntheticFunction* function)
 	uint32_t originalExceptionSize=mi->body->exceptions.size();
 	for(uint32_t i=0;i<originalExceptionSize;i++)
 	{
-		exception_info* ei=&mi->body->exceptions[i];
+		exception_info_abc* ei=&mi->body->exceptions[i];
 		//Find out where the exception begins
 		uint32_t excStart = ei->from;
 		assert(instructionsMap.find(ei->target)!=instructionsMap.end());
