@@ -20,8 +20,9 @@
 #include <stack>
 #include "scripting/flash/display/BitmapContainer.h"
 #include "scripting/flash/display/flashdisplay.h"
-#include "backends/rendering_context.h"
+#include "backends/rendering.h"
 #include "backends/image.h"
+#include "swf.h"
 
 using namespace std;
 using namespace lightspark;
@@ -31,6 +32,18 @@ BitmapContainer::BitmapContainer(MemoryAccount* m):stride(0),width(0),height(0),
 {
 }
 
+BitmapContainer::~BitmapContainer()
+{
+        if (bitmaptexture.isValid())
+        {
+                RenderThread* rt = getSys()->getRenderThread();
+                if (rt)
+                {
+                        getSys()->getRenderThread()->releaseTexture(bitmaptexture);
+                }
+        }
+}
+
 bool BitmapContainer::fromRGB(uint8_t* rgb, uint32_t w, uint32_t h, BITMAP_FORMAT format, bool frompng)
 {
 	if(!rgb)
@@ -118,6 +131,35 @@ void BitmapContainer::clear()
 	stride=0;
 	width=0;
 	height=0;
+	bitmaptexture.makeEmpty();
+}
+
+void BitmapContainer::upload(uint8_t *data, uint32_t w, uint32_t h)
+{
+	memcpy(data, getData(), w*h*4);
+}
+
+const TextureChunk &BitmapContainer::getTexture()
+{
+	return bitmaptexture;
+}
+
+void BitmapContainer::uploadFence()
+{
+	decRef();// is increffed in checkTexture
+}
+bool BitmapContainer::checkTexture()
+{
+    if (isEmpty()) {
+        return false;
+    }
+
+	if (!bitmaptexture.isValid())
+	{
+		bitmaptexture=getSys()->getRenderThread()->allocateTexture(width, height, true);
+	}
+	incRef();// is decreffed in uploadFence
+    return true;
 }
 
 void BitmapContainer::setAlpha(int32_t x, int32_t y, uint8_t alpha)
@@ -196,7 +238,6 @@ void BitmapContainer::copyRectangle(_R<BitmapContainer> source,
 
 	if (copyWidth <= 0 || copyHeight <= 0)
 		return;
-
 	int sx = clippedSourceRect.Xmin;
 	int sy = clippedSourceRect.Ymin;
 	if (mergeAlpha==false)
