@@ -26,7 +26,7 @@
 #include "scripting/flash/errors/flasherrors.h"
 #include "scripting/flash/utils/ByteArray.h"
 #include "scripting/flash/filters/flashfilters.h"
-#include "backends/rendering_context.h"
+#include "backends/rendering.h"
 #include "3rdparty/perlinnoise/PerlinNoise.hpp"
 
 #include <cstdlib> 
@@ -99,10 +99,10 @@ void BitmapData::sinit(Class_base* c)
 	c->setDeclaredMethodByQName("merge","",Class<IFunction>::getFunction(c->getSystemState(),threshold),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("paletteMap","",Class<IFunction>::getFunction(c->getSystemState(),paletteMap),NORMAL_METHOD,true);
 	// properties
-	c->setDeclaredMethodByQName("height","",Class<IFunction>::getFunction(c->getSystemState(),_getHeight),GETTER_METHOD,true);
-	c->setDeclaredMethodByQName("rect","",Class<IFunction>::getFunction(c->getSystemState(),getRect),GETTER_METHOD,true);
-	c->setDeclaredMethodByQName("width","",Class<IFunction>::getFunction(c->getSystemState(),_getWidth),GETTER_METHOD,true);
-	REGISTER_GETTER(c,transparent);
+	c->setDeclaredMethodByQName("height","",Class<IFunction>::getFunction(c->getSystemState(),_getHeight,0,Class<Integer>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("rect","",Class<IFunction>::getFunction(c->getSystemState(),getRect,0,Class<Rectangle>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("width","",Class<IFunction>::getFunction(c->getSystemState(),_getWidth,0,Class<Integer>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
+	REGISTER_GETTER_RESULTTYPE(c,transparent,Boolean);
 
 	IBitmapDrawable::linkTraits(c);
 }
@@ -110,6 +110,14 @@ void BitmapData::sinit(Class_base* c)
 void BitmapData::addUser(Bitmap* b)
 {
 	users.insert(b);
+	if (!pixels.isNull())
+	{
+		if (pixels->checkTexture())
+		{
+			getSystemState()->getRenderThread()->addUploadJob(this->pixels.getPtr());
+		}
+	}
+	b->updatedData();
 }
 
 void BitmapData::removeUser(Bitmap* b)
@@ -119,9 +127,16 @@ void BitmapData::removeUser(Bitmap* b)
 
 void BitmapData::notifyUsers() const
 {
-	if (locked > 0)
+	if (locked > 0 || users.empty())
 		return;
 
+	if (!pixels.isNull())
+	{
+		if (pixels->checkTexture())
+        {
+		    getSystemState()->getRenderThread()->addUploadJob(this->pixels.getPtr());
+        }
+	}
 	for(auto it=users.begin();it!=users.end();it++)
 		(*it)->updatedData();
 }
@@ -188,18 +203,44 @@ void BitmapData::drawDisplayObject(DisplayObject* d, const MATRIX& initialMatrix
 	d->hasChanged=true;
 	d->requestInvalidation(&queue);
 	CairoRenderContext ctxt(pixels->getData(), pixels->getWidth(), pixels->getHeight(),smoothing);
+	map<uint32_t,pair<IDrawable*,uint8_t*>> drawablecache;
 	for(auto it=queue.queue.begin();it!=queue.queue.end();it++)
 	{
 		DisplayObject* target=(*it).getPtr();
+		if (target->legacy)
+		{
+			auto it2 = drawablecache.find(target->getTagID());
+			if (it2 != drawablecache.end())
+			{
+				CachedSurface& surface=ctxt.allocateCustomSurface(target,(*it2).second.second,false);
+				IDrawable* dr = (*it2).second.first;
+				surface.tex->width=dr->getWidth();
+				surface.tex->height=dr->getHeight();
+				surface.xOffset=dr->getXOffset();
+				surface.yOffset=dr->getYOffset();
+				surface.xOffsetTransformed=dr->getXOffsetTransformed();
+				surface.yOffsetTransformed=dr->getYOffsetTransformed();
+				surface.widthTransformed=dr->getWidthTransformed();
+				surface.heightTransformed=dr->getHeightTransformed();
+				surface.rotation=dr->getRotation();
+				surface.xscale = dr->getXScale();
+				surface.yscale = dr->getYScale();
+				continue;
+			}
+		}
 		//Get the drawable from each of the added objects
 		IDrawable* drawable=target->invalidate(d, initialMatrix,smoothing);
 		if(drawable==nullptr)
 			continue;
 
 		//Compute the matrix for this object
-		uint8_t* buf=drawable->getPixelBuffer(initialMatrix.getScaleX(),initialMatrix.getScaleY());
+		bool isBufferOwner=true;
+		uint8_t* buf=drawable->getPixelBuffer(initialMatrix.getScaleX(),initialMatrix.getScaleY(),&isBufferOwner);
+		if (target->legacy)
+			drawablecache[target->getTagID()]=make_pair(drawable,buf);
+		
 		//Construct a CachedSurface using the data
-		CachedSurface& surface=ctxt.allocateCustomSurface(target,buf);
+		CachedSurface& surface=ctxt.allocateCustomSurface(target,buf,isBufferOwner);
 		surface.tex->width=drawable->getWidth();
 		surface.tex->height=drawable->getHeight();
 		surface.xOffset=drawable->getXOffset();
@@ -211,7 +252,14 @@ void BitmapData::drawDisplayObject(DisplayObject* d, const MATRIX& initialMatrix
 		surface.rotation=drawable->getRotation();
 		surface.xscale = drawable->getXScale();
 		surface.yscale = drawable->getYScale();
-		delete drawable;
+		if (!target->legacy)
+			delete drawable;
+	}
+	auto it3 = drawablecache.begin();
+	while (it3 != drawablecache.end())
+	{
+		delete (*it3).second.first;
+		it3++;
 	}
 	d->Render(ctxt,true);
 }
@@ -403,7 +451,7 @@ ASFUNCTIONBODY_ATOM(BitmapData,copyPixels)
 
 	th->pixels->copyRectangle(source->pixels, sourceRect->getRect(),
 				  destPoint->getX(), destPoint->getY(),
-				  mergeAlpha);
+				  mergeAlpha|| !th->transparent);
 	th->notifyUsers();
 }
 
