@@ -53,35 +53,43 @@ public:
 	/*
 	 * Utility method to draw a DisplayObject on the surface
 	 */
-	void drawDisplayObject(DisplayObject* d, const MATRIX& initialMatrix);
+	void drawDisplayObject(DisplayObject* d, const MATRIX& initialMatrix,bool smoothing);
 	ASPROPERTY_GETTER(bool, transparent);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(dispose);
-	ASFUNCTION(draw);
-	ASFUNCTION(getPixel);
-	ASFUNCTION(getPixel32);
-	ASFUNCTION(setPixel);
-	ASFUNCTION(setPixel32);
-	ASFUNCTION(getRect);
-	ASFUNCTION(copyPixels);
-	ASFUNCTION(fillRect);
-	ASFUNCTION(generateFilterRect);
-	ASFUNCTION(hitTest);
-	ASFUNCTION(_getWidth);
-	ASFUNCTION(_getHeight);
-	ASFUNCTION(scroll);
-	ASFUNCTION(clone);
-	ASFUNCTION(copyChannel);
-	ASFUNCTION(lock);
-	ASFUNCTION(unlock);
-	ASFUNCTION(floodFill);
-	ASFUNCTION(histogram);
-	ASFUNCTION(getColorBoundsRect);
-	ASFUNCTION(getPixels);
-	ASFUNCTION(getVector);
-	ASFUNCTION(setPixels);
-	ASFUNCTION(setVector);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(dispose);
+	ASFUNCTION_ATOM(draw);
+	ASFUNCTION_ATOM(getPixel);
+	ASFUNCTION_ATOM(getPixel32);
+	ASFUNCTION_ATOM(setPixel);
+	ASFUNCTION_ATOM(setPixel32);
+	ASFUNCTION_ATOM(getRect);
+	ASFUNCTION_ATOM(copyPixels);
+	ASFUNCTION_ATOM(fillRect);
+	ASFUNCTION_ATOM(generateFilterRect);
+	ASFUNCTION_ATOM(hitTest);
+	ASFUNCTION_ATOM(_getWidth);
+	ASFUNCTION_ATOM(_getHeight);
+	ASFUNCTION_ATOM(scroll);
+	ASFUNCTION_ATOM(clone);
+	ASFUNCTION_ATOM(copyChannel);
+	ASFUNCTION_ATOM(lock);
+	ASFUNCTION_ATOM(unlock);
+	ASFUNCTION_ATOM(floodFill);
+	ASFUNCTION_ATOM(histogram);
+	ASFUNCTION_ATOM(getColorBoundsRect);
+	ASFUNCTION_ATOM(getPixels);
+	ASFUNCTION_ATOM(getVector);
+	ASFUNCTION_ATOM(setPixels);
+	ASFUNCTION_ATOM(setVector);
+	ASFUNCTION_ATOM(colorTransform);
+	ASFUNCTION_ATOM(compare);
+	ASFUNCTION_ATOM(applyFilter);
+	ASFUNCTION_ATOM(noise);
+	ASFUNCTION_ATOM(perlinNoise);
+	ASFUNCTION_ATOM(threshold);
+	ASFUNCTION_ATOM(merge);
+	ASFUNCTION_ATOM(paletteMap);
 };
 
-};
+}
 #endif /* SCRIPTING_FLASH_DISPLAY_BITMAPDATA_H */
