@@ -113,10 +113,12 @@ public:
 
 class ColorTransform: public ASObject
 {
+friend class Bitmap;
 friend class BitmapData;
 friend class DisplayObject;
 friend class AVM1Color;
 friend class TokenContainer;
+friend class TextField;
 protected:
 	number_t redMultiplier,greenMultiplier,blueMultiplier,alphaMultiplier;
 	number_t redOffset,greenOffset,blueOffset,alphaOffset;
