@@ -46,6 +46,8 @@ protected:
 	Downloader* downloader;
 	_R<StreamCache> soundData;
 	_NR<SoundChannel> soundChannel;
+	StreamDecoder* rawDataStreamDecoder;
+	int32_t rawDataStartPosition;
 	// If container is true, audio format is parsed from
 	// soundData. If container is false, soundData is raw samples
 	// and format is defined by format member.
@@ -67,6 +69,7 @@ public:
 	ASFUNCTION_ATOM(load);
 	ASFUNCTION_ATOM(play);
 	ASFUNCTION_ATOM(close);
+	ASFUNCTION_ATOM(extract);
 	ASFUNCTION_ATOM(loadCompressedDataFromByteArray);
 	void afterExecution(_R<Event> e);
 };
@@ -99,6 +102,7 @@ private:
 	void validateSoundTransform(_NR<SoundTransform>);
 	void playStream();
 	number_t startTime;
+	int32_t loopstogo;
 	bool restartafterabort;
 public:
 	SoundChannel(Class_base* c, _NR<StreamCache> stream=NullRef, AudioFormat format=AudioFormat(CODEC_NONE,0,0), bool autoplay=true);
@@ -108,6 +112,7 @@ public:
 	void resume();
 	void markFinished(); // indicates that all sound data is available
 	void setStartTime(number_t starttime) { startTime = starttime; }
+	void setLoops(int32_t loops) {loopstogo=loops;}
 	static void sinit(Class_base* c);
 	static void buildTraits(ASObject* o);
 	void finalize();
@@ -134,14 +139,17 @@ private:
 	_NR<NetStream> netStream;
 	ASPROPERTY_GETTER_SETTER(int32_t, deblocking);
 	ASPROPERTY_GETTER_SETTER(bool, smoothing);
-	VideoDecoder* embeddedVideoDecoder;
 	DefineVideoStreamTag* videotag;
-	uint32_t embeddedframesbuffered;
+	VideoDecoder* embeddedVideoDecoder;
+	uint32_t lastuploadedframe;
+	void resetDecoder();
 public:
 	Video(Class_base* c, uint32_t w=320, uint32_t h=240, DefineVideoStreamTag* v=nullptr);
 	bool destruct() override;
-	void setVideoFrame(uint32_t FrameNum,uint8_t* framedata,uint32_t numbytes);
-	void checkRatio(uint32_t ratio) override;
+	void finalize() override;
+	void checkRatio(uint32_t ratio, bool inskipping) override;
+	void afterLegacyDelete(DisplayObjectContainer* par) override;
+	void setOnStage(bool staged, bool force = false) override;
 	uint32_t getTagID() const override;
 	~Video();
 	static void sinit(Class_base*);
