@@ -10,8 +10,7 @@ URLRequestHeader::URLRequestHeader(Class_base* c) : ASObject(c)
 
 void URLRequestHeader::sinit(Class_base* c)
 {
-	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->setSuper(Class<ASObject>::getRef());
+	CLASS_SETUP(c, ASObject, _constructor, CLASS_FINAL | CLASS_SEALED);
 	REGISTER_GETTER_SETTER(c,name);
 	REGISTER_GETTER_SETTER(c,value);
 }
@@ -20,11 +19,10 @@ void URLRequestHeader::buildTraits(ASObject* o)
 {
 }
 
-ASFUNCTIONBODY(URLRequestHeader,_constructor)
+ASFUNCTIONBODY_ATOM(URLRequestHeader,_constructor)
 {
-	URLRequestHeader* th=Class<URLRequestHeader>::cast(obj);
-	ARG_UNPACK (th->name, "") (th->value, "");
-	return NULL;
+	URLRequestHeader* th=asAtomHandler::as<URLRequestHeader>(obj);
+	ARG_UNPACK_ATOM (th->name, "") (th->value, "");
 }
 
 ASFUNCTIONBODY_GETTER_SETTER(URLRequestHeader,name);
