@@ -21,7 +21,6 @@
 #define SCRIPTING_FLASH_NET_URLREQUESTHEADER_H 1
 
 #include "asobject.h"
-#include "scripting/class.h"
 #include "tiny_string.h"
 
 namespace lightspark
@@ -33,7 +32,7 @@ public:
 	URLRequestHeader(Class_base* c);
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
-	ASFUNCTION(_constructor);
+	ASFUNCTION_ATOM(_constructor);
 	ASPROPERTY_GETTER_SETTER(tiny_string, name);
 	ASPROPERTY_GETTER_SETTER(tiny_string, value);
 };
