@@ -25,20 +25,17 @@
 #include "argconv.h"
 #include "swf.h"
 #include "flash/errors/flasherrors.h"
-#include <sys/types.h>
+
 #ifdef _WIN32
+#ifndef _WIN32_WINNT
 #	define _WIN32_WINNT 0x0501
+#endif
 #	include <winsock2.h>
 #	include <ws2tcpip.h>
 #	include <fcntl.h>
 #else
-#	include <sys/socket.h>
-#	include <netdb.h>
-#	include <sys/select.h>
-#endif
-#include <string.h>
 #include <unistd.h>
-#include <errno.h>
+#endif
 
 const char SOCKET_COMMAND_SEND = '*';
 const char SOCKET_COMMAND_CLOSE = '-';
@@ -46,138 +43,17 @@ const char SOCKET_COMMAND_CLOSE = '-';
 using namespace std;
 using namespace lightspark;
 
-SocketIO::SocketIO() : fd(-1)
-{
-#ifdef _WIN32
-	//TODO: move WSAStartup/WSACleanup to some more global place
-	WSADATA wsdata;
-	if(WSAStartup(MAKEWORD(2, 2), &wsdata))
-		LOG(LOG_ERROR,"WSAStartup failed");
-#endif
-}
-
-SocketIO::~SocketIO()
-{
-	if (fd != -1)
-		::close(fd);
-#ifdef _WIN32
-	WSACleanup();
-#endif
-}
-
-bool SocketIO::connect(const tiny_string& hostname, int port)
-{
-	struct addrinfo hints;
-	struct addrinfo *servinfo;
-	struct addrinfo *p;
-
-	if (fd != -1)
-		return false;
-	fd = -1;
-
-	if (port <= 0 || port > 65535)
-		return false;
-
-	memset(&hints, 0, sizeof hints);
-	hints.ai_family = AF_UNSPEC;
-	hints.ai_socktype = SOCK_STREAM;
-
-	tiny_string portstr = Integer::toString(port);
-	if (getaddrinfo(hostname.raw_buf(), portstr.raw_buf(), &hints, &servinfo) != 0)
-	{
-		return false;
-	}
-
-	for(p = servinfo; p != NULL; p = p->ai_next)
-	{
-		if ((fd = socket(p->ai_family, p->ai_socktype, p->ai_protocol)) == -1)
-			continue;
-
-		// TODO: timeout on connect
-
-		if (::connect(fd, p->ai_addr, p->ai_addrlen) == -1)
-		{
-			::close(fd);
-			continue;
-		}
-
-		break;
-	}
-
-	freeaddrinfo(servinfo);
-
-	if (!p)
-		fd = -1;
-
-	return fd != -1;
-}
-
-bool SocketIO::connected() const
-{
-	return fd != -1;
-}
-
-void SocketIO::close()
-{
-	if (fd != -1)
-	{
-		::close(fd);
-		fd = -1;
-	}
-}
-
-ssize_t SocketIO::receive(void *buf, size_t count) const
-{
-	ssize_t n;
-
-	do
-	{
-		n = read(fd, buf, count);
-	}
-	while (n < 0 && errno == EINTR);
-
-	return n;
-}
-
-ssize_t SocketIO::sendAll(const void *buf, size_t count) const
-{
-	ssize_t n;
-	ssize_t total;
-
-	char *writeptr = (char *)buf;
-	total = 0;
-	while (count > 0) {
-		n = write(fd, writeptr, count);
-
-		if (n < 0)
-		{
-			if (errno == EINTR)
-				continue;
-
-			return -1;
-		}
-
-		writeptr += n;
-		total += n;
-		count -= n;
-	}
-
-	return total;
-}
-
 XMLSocket::~XMLSocket()
 {
-	finalize();
 }
 
 void XMLSocket::sinit(Class_base* c)
 {
-	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->setSuper(Class<EventDispatcher>::getRef());
-	c->setDeclaredMethodByQName("close","",Class<IFunction>::getFunction(_close),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("connect","",Class<IFunction>::getFunction(_connect),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("send","",Class<IFunction>::getFunction(_send),NORMAL_METHOD,true);
-	c->setDeclaredMethodByQName("connected","",Class<IFunction>::getFunction(_connected),GETTER_METHOD,true);
+	CLASS_SETUP(c, EventDispatcher, _constructor, CLASS_SEALED);
+	c->setDeclaredMethodByQName("close","",Class<IFunction>::getFunction(c->getSystemState(),_close),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("connect","",Class<IFunction>::getFunction(c->getSystemState(),_connect),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("send","",Class<IFunction>::getFunction(c->getSystemState(),_send),NORMAL_METHOD,true);
+	c->setDeclaredMethodByQName("connected","",Class<IFunction>::getFunction(c->getSystemState(),_connected),GETTER_METHOD,true);
 	REGISTER_GETTER_SETTER(c,timeout);
 }
 
@@ -189,7 +65,7 @@ void XMLSocket::finalize()
 {
 	EventDispatcher::finalize();
 
-	SpinlockLocker l(joblock);
+	Locker l(joblock);
 	if (job)
 	{
 		job->threadAborting = true;
@@ -197,21 +73,22 @@ void XMLSocket::finalize()
 		job->threadAbort();
 		job = NULL;
 	}
+	timeout = 20000;
 }
 
 ASFUNCTIONBODY_GETTER_SETTER(XMLSocket, timeout);
 
-ASFUNCTIONBODY(XMLSocket, _constructor)
+ASFUNCTIONBODY_ATOM(XMLSocket,_constructor)
 {
 	tiny_string host;
 	bool host_is_null;
 	int port;
-	ARG_UNPACK (host, "") (port, 0);
+	ARG_UNPACK_ATOM (host, "") (port, 0);
 
-	EventDispatcher::_constructor(obj,NULL,0);
+	EventDispatcher::_constructor(ret,sys,obj,NULL,0);
 
-	XMLSocket* th=obj->as<XMLSocket>();
-	host_is_null = argslen > 0 && args[0]->is<Null>();
+	XMLSocket* th=asAtomHandler::as<XMLSocket>(obj);
+	host_is_null = argslen > 0 && asAtomHandler::is<Null>(args[0]);
 	if (port != 0)
 	{
 		if (host_is_null)
@@ -219,37 +96,33 @@ ASFUNCTIONBODY(XMLSocket, _constructor)
 		else if (!host.empty())
 			th->connect(host, port);
 	}
-
-	return NULL;
 }
 
-ASFUNCTIONBODY(XMLSocket, _close)
+ASFUNCTIONBODY_ATOM(XMLSocket, _close)
 {
-	XMLSocket* th=obj->as<XMLSocket>();
-	SpinlockLocker l(th->joblock);
+	XMLSocket* th=asAtomHandler::as<XMLSocket>(obj);
+	Locker l(th->joblock);
 
 	if (th->job)
 	{
 		th->job->requestClose();
 	}
-
-	return NULL;
 }
 
 void XMLSocket::connect(tiny_string host, int port)
 {
 	if (port <= 0 || port > 65535)
-		throw Class<SecurityError>::getInstanceS("Invalid port");
+		throw Class<SecurityError>::getInstanceS(getSystemState(),"Invalid port");
 
 	if (host.empty())
 		host = getSys()->mainClip->getOrigin().getHostname();
 
 	if (isConnected())
-		throw Class<IOError>::getInstanceS("Already connected");
+		throw Class<IOError>::getInstanceS(getSystemState(),"Already connected");
 
 	// Host shouldn't contain scheme or port
 	if (host.strchr(':') != NULL)
-		throw Class<SecurityError>::getInstanceS("Invalid hostname");
+		throw Class<SecurityError>::getInstanceS(getSystemState(),"Invalid hostname");
 
 	// Check sandbox and policy file
 	size_t buflen = host.numBytes() + 22;
@@ -257,7 +130,7 @@ void XMLSocket::connect(tiny_string host, int port)
 	snprintf(urlbuf, buflen, "xmlsocket://%s:%d", host.raw_buf(), port);
 	URLInfo url(urlbuf);
 
-	getSys()->securityManager->checkURLStaticAndThrow(url,
+	getSystemState()->securityManager->checkURLStaticAndThrow(url,
 		~(SecurityManager::LOCAL_WITH_FILE),
 		SecurityManager::LOCAL_WITH_FILE | SecurityManager::LOCAL_TRUSTED,
 		true);
@@ -267,7 +140,7 @@ void XMLSocket::connect(tiny_string host, int port)
 	if(evaluationResult != SecurityManager::ALLOWED)
 	{
 		incRef();
-		getVm()->addEvent(_MR(this), _MR(Class<SecurityErrorEvent>::getInstanceS("No policy file allows socket connection")));
+		getVm(getSystemState())->addEvent(_MR(this), _MR(Class<SecurityErrorEvent>::getInstanceS(getSystemState(),"No policy file allows socket connection")));
 		return;
 	}
 
@@ -277,57 +150,53 @@ void XMLSocket::connect(tiny_string host, int port)
 	job = thread;
 }
 
-ASFUNCTIONBODY(XMLSocket, _connect)
+ASFUNCTIONBODY_ATOM(XMLSocket, _connect)
 {
-	XMLSocket* th=obj->as<XMLSocket>();
+	XMLSocket* th=asAtomHandler::as<XMLSocket>(obj);
 	tiny_string host;
 	bool host_is_null;
 	int port;
-	ARG_UNPACK (host) (port);
-	host_is_null = argslen > 0 && args[0]->is<Null>();
+	ARG_UNPACK_ATOM (host) (port);
+	host_is_null = argslen > 0 && asAtomHandler::is<Null>(args[0]);
 
 	if (host_is_null)
 		th->connect("", port);
 	else
 		th->connect(host, port);
-
-	return NULL;
 }
 
-ASFUNCTIONBODY(XMLSocket, _send)
+ASFUNCTIONBODY_ATOM(XMLSocket, _send)
 {
-	XMLSocket* th=obj->as<XMLSocket>();
+	XMLSocket* th=asAtomHandler::as<XMLSocket>(obj);
 	tiny_string data;
-	ARG_UNPACK (data);
+	ARG_UNPACK_ATOM (data);
 
-	SpinlockLocker l(th->joblock);
+	Locker l(th->joblock);
 	if (th->job)
 	{
 		th->job->sendData(data);
 	}
 	else
 	{
-		throw Class<IOError>::getInstanceS("Socket is not connected");
+		throw Class<IOError>::getInstanceS(sys,"Socket is not connected");
 	}
-
-	return NULL;
 }
 
 bool XMLSocket::isConnected()
 {
-	SpinlockLocker l(joblock);
+	Locker l(joblock);
 	return job && job->isConnected();
 }
 
-ASFUNCTIONBODY(XMLSocket, _connected)
+ASFUNCTIONBODY_ATOM(XMLSocket, _connected)
 {
-	XMLSocket* th=obj->as<XMLSocket>();
-	return abstract_b(th->isConnected());
+	XMLSocket* th=asAtomHandler::as<XMLSocket>(obj);
+	asAtomHandler::setBool(ret,th->isConnected());
 }
 
 void XMLSocket::threadFinished()
 {
-	SpinlockLocker l(joblock);
+	Locker l(joblock);
 	job = NULL;
 }
 
@@ -379,11 +248,13 @@ void XMLSocketThread::execute()
 {
 	if (!sock.connect(hostname, port))
 	{
-		getVm()->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS()));
+		owner->incRef();
+		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getSystemState())));
 		return;
 	}
 
-	getVm()->addEvent(owner, _MR(Class<Event>::getInstanceS("connect")));
+	owner->incRef();
+	getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getSystemState(),"connect")));
 
 	struct timeval timeout;
 	int maxfd;
@@ -403,7 +274,8 @@ void XMLSocketThread::execute()
 		int status = select(maxfd+1, &readfds, NULL, NULL, &timeout);
 		if (status  < 0)
 		{
-			getVm()->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS()));
+			owner->incRef();
+			getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getSystemState())));
 			return;
 		}
 
@@ -414,7 +286,8 @@ void XMLSocketThread::execute()
 			ssize_t nbytes = read(signalListener, &cmd, 1);
 			if (nbytes < 0)
 			{
-				getVm()->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS()));
+				owner->incRef();
+				getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getSystemState())));
 				return;
 			}
 			else if (nbytes == 0)
@@ -445,18 +318,21 @@ void XMLSocketThread::readSocket(const SocketIO& sock)
 	{
 		buf[nbytes] = '\0';
 		tiny_string data(buf, true);
-		getVm()->addEvent(owner, _MR(Class<DataEvent>::getInstanceS(data)));
+		owner->incRef();
+		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<DataEvent>::getInstanceS(owner->getSystemState(),data)));
 	}
 	else if (nbytes == 0)
 	{
 		// The server has closed the socket
-		getVm()->addEvent(owner, _MR(Class<Event>::getInstanceS("close")));
+		owner->incRef();
+		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<Event>::getInstanceS(owner->getSystemState(),"close")));
 		threadAborting = true;
 	}
 	else
 	{
 		// Error
-		getVm()->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS()));
+		owner->incRef();
+		getVm(owner->getSystemState())->addEvent(owner, _MR(Class<IOErrorEvent>::getInstanceS(owner->getSystemState())));
 		threadAborting = true;
 	}
 }
@@ -508,10 +384,16 @@ void XMLSocketThread::sendData(const tiny_string& data)
 
 	tiny_string *packet = new tiny_string(data);
 	g_async_queue_push(sendQueue, packet);
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-result"
 	write(signalEmitter, &SOCKET_COMMAND_SEND, 1);
+#pragma GCC diagnostic pop
 }
 
 void XMLSocketThread::requestClose()
 {
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wunused-result"
 	write(signalEmitter, &SOCKET_COMMAND_CLOSE, 1);
+#pragma GCC diagnostic pop
 }
