@@ -29,19 +29,20 @@
 using namespace std;
 using namespace lightspark;
 
-Accelerometer::Accelerometer(Class_base* c):ASObject(c) {}
+Accelerometer::Accelerometer(Class_base* c):EventDispatcher(c) {}
 
 void Accelerometer::sinit(Class_base* c)
 {
+	CLASS_SETUP_NO_CONSTRUCTOR(c, EventDispatcher, CLASS_SEALED);
 	// properties
-	c->setDeclaredMethodByQName("isSupported", "", Class<IFunction>::getFunction(_isSupported),GETTER_METHOD,false);
+	c->setDeclaredMethodByQName("isSupported", "", Class<IFunction>::getFunction(c->getSystemState(),_isSupported),GETTER_METHOD,false);
 }
 
 void Accelerometer::buildTraits(ASObject *o)
 {
 }
 
-ASFUNCTIONBODY(Accelerometer,_isSupported)
+ASFUNCTIONBODY_ATOM(Accelerometer,_isSupported)
 {
-	return abstract_b(false);
+	asAtomHandler::setBool(ret,false);
 }
