@@ -35,35 +35,42 @@ class XMLNode: public ASObject
 friend class XML;
 protected:
 	_NR<XMLDocument> root;
-	xmlpp::Node* node;
-	tiny_string toString_priv(xmlpp::Node *outputNode);
-	xmlpp::Node *getParentNode();
+	pugi::xml_node node;
+	tiny_string toString_priv(pugi::xml_node outputNode);
+	pugi::xml_node getParentNode();
 public:
 	XMLNode(Class_base* c):ASObject(c),root(NullRef),node(NULL){}
-	XMLNode(Class_base* c, _R<XMLDocument> _r, xmlpp::Node* _n);
-	void finalize();
+	XMLNode(Class_base* c, _NR<XMLDocument> _r, pugi::xml_node _n);
+	bool destruct()
+	{
+		root.reset();
+		return destructIntern();
+	}
 	static void sinit(Class_base*);
 	static void buildTraits(ASObject* o);
 	tiny_string toString();
-	ASFUNCTION(_constructor);
-	ASFUNCTION(firstChild);
-	ASFUNCTION(lastChild);
-	ASFUNCTION(childNodes);
-	ASFUNCTION(attributes);
-	ASFUNCTION(_getNodeType);
-	ASFUNCTION(_getNodeName);
-	ASFUNCTION(_getNodeValue);
-	ASFUNCTION(nextSibling);
-	ASFUNCTION(parentNode);
-	ASFUNCTION(previousSibling);
-	ASFUNCTION(_toString);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(firstChild);
+	ASFUNCTION_ATOM(lastChild);
+	ASFUNCTION_ATOM(childNodes);
+	ASFUNCTION_ATOM(attributes);
+	ASFUNCTION_ATOM(_getNodeType);
+	ASFUNCTION_ATOM(_getNodeName);
+	ASFUNCTION_ATOM(_setNodeName);
+	ASFUNCTION_ATOM(_getNodeValue);
+	ASFUNCTION_ATOM(_getLocalName);
+	ASFUNCTION_ATOM(nextSibling);
+	ASFUNCTION_ATOM(parentNode);
+	ASFUNCTION_ATOM(previousSibling);
+	ASFUNCTION_ATOM(_toString);
+	ASFUNCTION_ATOM(appendChild);
 };
 
 class XMLDocument: public XMLNode, public XMLBase
 {
 friend class XMLNode;
 private:
-	xmlpp::Node* rootNode;
+	pugi::xml_node rootNode;
 public:
 	XMLDocument(Class_base* c, tiny_string s="");
 	void parseXMLImpl(const std::string& str);
@@ -71,10 +78,11 @@ public:
 	static void buildTraits(ASObject* o);
 	tiny_string toString();
 	ASPROPERTY_GETTER_SETTER(bool, ignoreWhite);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(parseXML);
-	ASFUNCTION(firstChild);
-	ASFUNCTION(_toString);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(parseXML);
+	ASFUNCTION_ATOM(firstChild);
+	ASFUNCTION_ATOM(_toString);
+	ASFUNCTION_ATOM(createElement);
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
