@@ -61,7 +61,7 @@ void Array::sinit(Class_base* c)
 	c->setVariableAtomByQName("UNIQUESORT",nsNameAndKind(),asAtomHandler::fromUInt(UNIQUESORT),CONSTANT_TRAIT);
 
 	// properties
-	c->setDeclaredMethodByQName("length","",Class<IFunction>::getFunction(c->getSystemState(),_getLength),GETTER_METHOD,true);
+	c->setDeclaredMethodByQName("length","",Class<IFunction>::getFunction(c->getSystemState(),_getLength,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,true);
 	c->setDeclaredMethodByQName("length","",Class<IFunction>::getFunction(c->getSystemState(),_setLength),SETTER_METHOD,true);
 
 	// public functions
@@ -87,7 +87,7 @@ void Array::sinit(Class_base* c)
 	c->setDeclaredMethodByQName("insertAt",AS3,Class<IFunction>::getFunction(c->getSystemState(),insertAt,2),NORMAL_METHOD,true);
 	c->setDeclaredMethodByQName("removeAt",AS3,Class<IFunction>::getFunction(c->getSystemState(),removeAt,1),NORMAL_METHOD,true);
 
-	c->prototype->getObj()->setDeclaredMethodByQName("length","",Class<IFunction>::getFunction(c->getSystemState(),_getLength),GETTER_METHOD,false);
+	c->prototype->getObj()->setDeclaredMethodByQName("length","",Class<IFunction>::getFunction(c->getSystemState(),_getLength,0,Class<UInteger>::getRef(c->getSystemState()).getPtr()),GETTER_METHOD,false);
 	c->prototype->getObj()->setDeclaredMethodByQName("length","",Class<IFunction>::getFunction(c->getSystemState(),_setLength),SETTER_METHOD,false);
 	c->prototype->setVariableByQName("concat","",Class<IFunction>::getFunction(c->getSystemState(),_concat,1),DYNAMIC_TRAIT);
 	c->prototype->setVariableByQName("every","",Class<IFunction>::getFunction(c->getSystemState(),every,1),DYNAMIC_TRAIT);
@@ -1640,6 +1640,11 @@ GET_VARIABLE_RESULT Array::getVariableByMultiname(asAtom& ret, const multiname&
 
 GET_VARIABLE_RESULT Array::getVariableByInteger(asAtom &ret, int index, GET_VARIABLE_OPTION opt)
 {
+	if (getClass() && getClass()->isSealed)
+	{
+		tiny_string s = Integer::toString(index);
+		throwError<ReferenceError>(kReadSealedError,s,getClass()->getQualifiedClassName());
+	}
 	if (index >=0 && uint32_t(index) < size())
 	{
 		if (index < ARRAY_SIZE_THRESHOLD)
@@ -1661,6 +1666,18 @@ GET_VARIABLE_RESULT Array::getVariableByInteger(asAtom &ret, int index, GET_VARI
 				ASATOM_INCREF(ret);
 			return GET_VARIABLE_RESULT::GETVAR_NORMAL;
 		}
+		//Check prototype chain
+		multiname m(nullptr);
+		m.name_type = multiname::NAME_INT;
+		m.name_i = index;
+		Prototype* proto = this->getClass()->prototype.getPtr();
+		while(proto && proto->getObj() != this)
+		{
+			GET_VARIABLE_RESULT res = proto->getObj()->getVariableByMultiname(ret,m, opt);
+			if(asAtomHandler::isValid(ret))
+				return res;
+			proto = proto->prevPrototype.getPtr();
+		}
 		asAtomHandler::setUndefined(ret);
 		return GET_VARIABLE_RESULT::GETVAR_NORMAL;
 	}
@@ -1668,7 +1685,7 @@ GET_VARIABLE_RESULT Array::getVariableByInteger(asAtom &ret, int index, GET_VARI
 		return getVariableByIntegerIntern(ret,index,opt);
 }
 
-void Array::setVariableByMultiname_i(const multiname& name, int32_t value)
+void Array::setVariableByMultiname_i(multiname& name, int32_t value)
 {
 	assert_and_throw(implEnable);
 	unsigned int index=0;
@@ -1760,7 +1777,7 @@ bool Array::isIntegerWithoutLeadingZeros(const tiny_string& value)
 	return true;
 }
 
-multiname *Array::setVariableByMultiname(const multiname& name, asAtom& o, CONST_ALLOWED_FLAG allowConst, bool* alreadyset)
+multiname *Array::setVariableByMultiname(multiname& name, asAtom& o, CONST_ALLOWED_FLAG allowConst, bool* alreadyset)
 {
 	assert_and_throw(implEnable);
 	uint32_t index=0;
