@@ -19,6 +19,7 @@
 
 #include "scripting/toplevel/Boolean.h"
 #include "scripting/toplevel/toplevel.h"
+#include "scripting/flash/utils/ByteArray.h"
 #include "scripting/class.h"
 #include "scripting/argconv.h"
 #include "parsing/amf3_generator.h"
@@ -26,14 +27,6 @@
 using namespace lightspark;
 using namespace std;
 
-Boolean* lightspark::abstract_b(bool v)
-{
-	if(v==true)
-		return getSys()->getTrueRef();
-	else
-		return getSys()->getFalseRef();
-}
-
 /* implements ecma3's ToBoolean() operation, see section 9.2, but returns the value instead of an Boolean object */
 bool lightspark::Boolean_concrete(const ASObject* o)
 {
@@ -45,13 +38,22 @@ bool lightspark::Boolean_concrete(const ASObject* o)
 	case T_BOOLEAN:
 		return o->as<Boolean>()->val;
 	case T_NUMBER:
-		return (o->as<Number>()->val != 0) && !std::isnan(o->as<Number>()->val);
+		return (o->as<Number>()->isfloat ? o->as<Number>()->dval != 0.0 && !std::isnan(o->as<Number>()->dval) : o->as<Number>()->ival != 0);
 	case T_INTEGER:
 		return o->as<Integer>()->val != 0;
 	case T_UINTEGER:
 		return o->as<UInteger>()->val != 0;
 	case T_STRING:
-		return !o->as<ASString>()->data.empty();
+		if (!o->isConstructed())
+			return false;
+		return !o->as<ASString>()->isEmpty();
+	case T_FUNCTION:
+	case T_ARRAY:
+	case T_OBJECT:
+		// not constructed objects return false
+		if (!o->isConstructed())
+			return false;
+		return true;
 	default:
 		//everything else is an Object regarding to the spec
 		return true;
@@ -59,68 +61,75 @@ bool lightspark::Boolean_concrete(const ASObject* o)
 }
 
 /* this implements the global Boolean() function */
-ASFUNCTIONBODY(Boolean,generator)
+ASFUNCTIONBODY_ATOM(Boolean,generator)
 {
 	if(argslen==1)
-		return abstract_b(Boolean_concrete(args[0]));
+		ret = asAtomHandler::fromBool(asAtomHandler::Boolean_concrete(args[0]));
 	else
-		return abstract_b(false);
+		ret = asAtomHandler::falseAtom;
 }
 
 void Boolean::sinit(Class_base* c)
 {
-	c->isFinal=true;
-	c->setConstructor(Class<IFunction>::getFunction(_constructor));
-	c->setSuper(Class<ASObject>::getRef());
-	c->prototype->setVariableByQName("toString",AS3,Class<IFunction>::getFunction(_toString),DYNAMIC_TRAIT);
-	c->prototype->setVariableByQName("valueOf",AS3,Class<IFunction>::getFunction(_valueOf),DYNAMIC_TRAIT);
+	CLASS_SETUP(c, ASObject, _constructor, CLASS_SEALED | CLASS_FINAL);
+	c->isReusable = true;
+	c->setDeclaredMethodByQName("toString",AS3,Class<IFunction>::getFunction(c->getSystemState(),_toString),NORMAL_METHOD,true);
+	c->prototype->setVariableByQName("toString","",Class<IFunction>::getFunction(c->getSystemState(),_toString),DYNAMIC_TRAIT);
+	c->prototype->setVariableByQName("valueOf","",Class<IFunction>::getFunction(c->getSystemState(),_valueOf),DYNAMIC_TRAIT);
 }
 
-ASFUNCTIONBODY(Boolean,_constructor)
+ASFUNCTIONBODY_ATOM(Boolean,_constructor)
 {
-	Boolean* th=static_cast<Boolean*>(obj);
+	Boolean* th=asAtomHandler::as<Boolean>(obj);
 	if(argslen==0)
 	{
 		//No need to handle default argument. The object is initialized to false anyway
-		return NULL;
+		return;
 	}
-	th->val=Boolean_concrete(args[0]);
-	return NULL;
+	th->val=asAtomHandler::Boolean_concrete(args[0]);
 }
 
-ASFUNCTIONBODY(Boolean,_toString)
+ASFUNCTIONBODY_ATOM(Boolean,_toString)
 {
-	if(Class<Boolean>::getClass()->prototype->getObj() == obj) //See ECMA 15.6.4
-		return Class<ASString>::getInstanceS("false");
+	if(Class<Boolean>::getClass(sys)->prototype->getObj() == asAtomHandler::getObject(obj)) //See ECMA 15.6.4
+	{
+		ret = asAtomHandler::fromString(sys,"false");
+		return;
+	}
 
-	if(!obj->is<Boolean>())
-		throw Class<TypeError>::getInstanceS("");
+	if(!asAtomHandler::is<Boolean>(obj))
+		throw Class<TypeError>::getInstanceS(sys,"");
 
-	Boolean* th=static_cast<Boolean*>(obj);
-	return Class<ASString>::getInstanceS(th->toString());
+	ret = asAtomHandler::fromString(sys,asAtomHandler::toString(obj,sys));
 }
 
-ASFUNCTIONBODY(Boolean,_valueOf)
+ASFUNCTIONBODY_ATOM(Boolean,_valueOf)
 {
-	if(Class<Boolean>::getClass()->prototype->getObj() == obj)
-		return abstract_b(false);
+	if(Class<Boolean>::getClass(sys)->prototype->getObj() == asAtomHandler::getObject(obj))
+	{
+		asAtomHandler::setBool(ret,false);
+		return;
+	}
 
-	if(!obj->is<Boolean>())
-			throw Class<TypeError>::getInstanceS("");
+	if(!asAtomHandler::is<Boolean>(obj))
+			throw Class<TypeError>::getInstanceS(sys,"");
 
 	//The ecma3 spec is unspecific, but testing showed that we should return
 	//a new object
-	return abstract_b(obj->as<Boolean>()->val);
+	asAtomHandler::setBool(ret,asAtomHandler::Boolean_concrete(obj));
 }
 
 void Boolean::serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
 				std::map<const Class_base*, uint32_t>& traitsMap)
 {
-	if(val)
-		out->writeByte(true_marker);
+	if (out->getObjectEncoding() == ObjectEncoding::AMF0)
+	{
+		out->writeByte(amf0_boolean_marker);
+		out->writeByte(val ? 1:0);
+	}
 	else
-		out->writeByte(false_marker);
+		out->writeByte(val ? true_marker : false_marker);
 }
 
 bool Boolean::isEqual(ASObject* r)
@@ -133,6 +142,9 @@ bool Boolean::isEqual(ASObject* r)
 			return b->val==val;
 		}
 		case T_STRING:
+			if (!r->isConstructed())
+				return false;
+			return val==r->toNumber();
 		case T_INTEGER:
 		case T_UINTEGER:
 		case T_NUMBER:
@@ -147,32 +159,81 @@ bool Boolean::isEqual(ASObject* r)
 
 TRISTATE Boolean::isLess(ASObject* r)
 {
-	if(r->getObjectType()==T_BOOLEAN)
+	switch (r->getObjectType())
 	{
-		const Boolean* b=static_cast<const Boolean*>(r);
-		return (val<b->val)?TTRUE:TFALSE;
+		case T_BOOLEAN:
+		{
+			const Boolean* b=static_cast<const Boolean*>(r);
+			return (val<b->val)?TTRUE:TFALSE;
+		}
+		case T_INTEGER:
+		{
+			int32_t d=r->toInt();
+			return (val<d)?TTRUE:TFALSE;
+		}
+		case T_UINTEGER:
+		{
+			uint32_t d=r->toUInt();
+			return (val<d)?TTRUE:TFALSE;
+		}
+		case T_NUMBER:
+		case T_STRING:
+		{
+			double d=r->toNumber();
+			if(std::isnan(d)) return TUNDEFINED;
+			return (val<d)?TTRUE:TFALSE;
+		}
+		case T_NULL:
+			return (val)?TFALSE:TTRUE;
+		case T_UNDEFINED:
+			return TUNDEFINED;
+		default:
+		{
+			asAtom val2p=asAtomHandler::invalidAtom;
+			r->toPrimitive(val2p);
+			double val2=asAtomHandler::toNumber(val2p);
+			if(std::isnan(val2)) return TUNDEFINED;
+			return (val<val2)?TTRUE:TFALSE;
+		}
 	}
-	else if(r->getObjectType()==T_INTEGER ||
-		r->getObjectType()==T_UINTEGER ||
-		r->getObjectType()==T_NUMBER ||
-		r->getObjectType()==T_STRING)
+}
+
+TRISTATE Boolean::isLessAtom(asAtom& r)
+{
+	switch (asAtomHandler::getObjectType(r))
 	{
-		double d=r->toNumber();
-		if(std::isnan(d)) return TUNDEFINED;
-		return (val<d)?TTRUE:TFALSE;
-	}
-	else if(r->getObjectType()==T_NULL)
-	{
-		return (val<0)?TTRUE:TFALSE;
-	}
-	else if(r->getObjectType()==T_UNDEFINED)
-	{
-		return TUNDEFINED;
-	}
-	else
-	{
-		double val2=r->toPrimitive()->toNumber();
-		if(std::isnan(val2)) return TUNDEFINED;
-		return (val<val2)?TTRUE:TFALSE;
+		case T_BOOLEAN:
+		{
+			return (val<asAtomHandler::toInt(r))?TTRUE:TFALSE;
+		}
+		case T_INTEGER:
+		{
+			int32_t d=asAtomHandler::toInt(r);
+			return (val<d)?TTRUE:TFALSE;
+		}
+		case T_UINTEGER:
+		{
+			uint32_t d=asAtomHandler::toUInt(r);
+			return (val<d)?TTRUE:TFALSE;
+		}
+		case T_NUMBER:
+		case T_STRING:
+		{
+			double d=asAtomHandler::toNumber(r);
+			if(std::isnan(d)) return TUNDEFINED;
+			return (val<d)?TTRUE:TFALSE;
+		}
+		case T_NULL:
+			return (val)?TFALSE:TTRUE;
+		case T_UNDEFINED:
+			return TUNDEFINED;
+		default:
+		{
+			asAtom val2p=asAtomHandler::invalidAtom;
+			asAtomHandler::getObject(r)->toPrimitive(val2p);
+			double val2=asAtomHandler::toNumber(val2p);
+			if(std::isnan(val2)) return TUNDEFINED;
+			return (val<val2)?TTRUE:TFALSE;
+		}
 	}
 }
