@@ -1,7 +1,7 @@
 /**************************************************************************
     Lightspark, a free flash player implementation
 
-    Copyright (C) 2009-2012  Alessandro Pignotti (a.pignotti@sssup.it)
+    Copyright (C) 2009-2013  Alessandro Pignotti (a.pignotti@sssup.it)
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -25,30 +25,32 @@
 namespace lightspark
 {
 
-/* returns a fully inialized Boolean object with value b
- * like Class<Boolean>::getInstanceS(b) but without the constructor problems */
-Boolean* abstract_b(bool b);
-
 /* implements ecma3's ToBoolean() operation, see section 9.2, but returns the value instead of an Boolean object */
 bool Boolean_concrete(const ASObject* obj);
 
 class Boolean: public ASObject
 {
 public:
-	Boolean(Class_base* c, bool v=false):ASObject(c),val(v) {type=T_BOOLEAN;}
+	Boolean(Class_base* c, bool v=false):ASObject(c,T_BOOLEAN),val(v) {}
 	static void sinit(Class_base*);
-	static void buildTraits(ASObject* o){};
+	static void buildTraits(ASObject* o){}
 	bool val;
+	inline bool destruct() { val=false; return destructIntern(); }
 	int32_t toInt()
 	{
 		return val ? 1 : 0;
 	}
+	int64_t toInt64()
+	{
+		return val ? 1 : 0;
+	}
 	bool isEqual(ASObject* r);
 	TRISTATE isLess(ASObject* r);
-	ASFUNCTION(_constructor);
-	ASFUNCTION(_toString);
-	ASFUNCTION(_valueOf);
-	ASFUNCTION(generator);
+	TRISTATE isLessAtom(asAtom& r);
+	ASFUNCTION_ATOM(_constructor);
+	ASFUNCTION_ATOM(_toString);
+	ASFUNCTION_ATOM(_valueOf);
+	ASFUNCTION_ATOM(generator);
 	//Serialization interface
 	void serialize(ByteArray* out, std::map<tiny_string, uint32_t>& stringMap,
 				std::map<const ASObject*, uint32_t>& objMap,
