@@ -54,6 +54,7 @@ class PluginManager;
 class RenderThread;
 class SecurityManager;
 class LocaleManager;
+class CurrencyManager;
 class Tag;
 class ApplicationDomain;
 class ASWorker;
@@ -74,7 +75,7 @@ private:
 	RGB Background;
 	Mutex dictSpinlock;
 	std::unordered_map < uint32_t, DictionaryTag* > dictionary;
-	std::list< std::pair<tiny_string, DictionaryTag*> > classesToBeBound;
+	std::map < QName, DictionaryTag* > classesToBeBound;
 	std::map < tiny_string,FontTag* > embeddedfonts;
 	std::map < uint32_t,FontTag* > embeddedfontsByID;
 
@@ -95,7 +96,9 @@ private:
 public:
 	RootMovieClip(_NR<LoaderInfo> li, _NR<ApplicationDomain> appDomain, _NR<SecurityDomain> secDomain, Class_base* c);
 	~RootMovieClip();
+	void destroyTags();
 	bool destruct() override;
+	void finalize() override;
 	bool hasFinishedLoading() override { return ACQUIRE_READ(finishedLoading); }
 	bool isWaitingForParser() { return waitingforparser; }
 	void constructionComplete() override;
@@ -238,10 +241,10 @@ private:
 #endif
 
 	//shared null, undefined, true and false instances
-	_NR<Null> null;
-	_NR<Undefined> undefined;
-	_NR<Boolean> trueRef;
-	_NR<Boolean> falseRef;
+	Null* null;
+	Undefined* undefined;
+	Boolean* trueRef;
+	Boolean* falseRef;
 	Class_base* objClassRef;
 
 	//Parameters/FlashVars
@@ -303,6 +306,8 @@ private:
 	Cond mainsignalCond;
 	void systemFinalize();
 	std::map<tiny_string, Class_base *> classnamemap;
+	set<ASObject*> constantrefs;
+	list<_R<DisplayObject>> listResetParent;
 public:
 	void setURL(const tiny_string& url) DLL_PUBLIC;
 	tiny_string getDumpedSWFPath() const { return dumpedSWFPath;}
@@ -336,6 +341,10 @@ public:
 	std::map<tiny_string, _R<SharedObject> > sharedobjectmap;
 	bool localStorageAllowed() const { return localstorageallowed; }
 	void setLocalStorageAllowed(bool allowed);
+	void registerConstantRef(ASObject* obj)
+	{
+		constantrefs.insert(obj);
+	}
 	void tick() override;
 	void tickFence() override;
 	RenderThread* getRenderThread() const { return renderThread; }
@@ -366,22 +375,22 @@ public:
 	
 	inline Null* getNullRef() const
 	{
-		return null.getPtr();
+		return null;
 	}
 	
 	inline Undefined* getUndefinedRef() const
 	{
-		return undefined.getPtr();
+		return undefined;
 	}
 	
 	inline Boolean* getTrueRef() const
 	{
-		return trueRef.getPtr();
+		return trueRef;
 	}
 	
 	inline Boolean* getFalseRef() const
 	{
-		return falseRef.getPtr();
+		return falseRef;
 	}
 
 	inline Class_base* getObjectClassRef() const
@@ -398,8 +407,9 @@ public:
 	//Application starting time in milliseconds
 	uint64_t startTime;
 
-	//Classes set. They own one reference to each class/template
-	std::set<Class_base*> customClasses;
+	//map of all classed defined in the swf. They own one reference to each class/template
+	//key is the stringID of the class name (without namespace)
+	std::multimap<uint32_t, Class_base*> customClasses;
 	//This is an array of fixed size, we can avoid using std::vector
 	Class_base** builtinClasses;
 	std::map<QName, Template_base*> templates;
@@ -440,10 +450,10 @@ public:
 	/*
 	 * The application domain for the system
 	 */
-	_NR<ApplicationDomain> systemDomain;
+	ApplicationDomain* systemDomain;
 
-	_NR<ASWorker> worker;
-	_NR<WorkerDomain> workerDomain;
+	ASWorker* worker;
+	WorkerDomain* workerDomain;
 	bool singleworker;
 	Mutex workerMutex;
 	void addWorker(ASWorker* w);
@@ -457,6 +467,7 @@ public:
 	IntervalManager* intervalManager;
 	SecurityManager* securityManager;
 	LocaleManager* localeManager;
+    CurrencyManager* currencyManager;
 	ExtScriptObject* extScriptObject;
 
 	enum SCALE_MODE { EXACT_FIT=0, NO_BORDER=1, NO_SCALE=2, SHOW_ALL=3 };
@@ -552,6 +563,48 @@ public:
 	Cond initializedCond;
 	void waitInitialized();
 	void getClassInstanceByName(asAtom &ret, const tiny_string& clsname);
+	Mutex resetParentMutex;
+	void addDisplayObjectToResetParentList(_R<DisplayObject> child)
+	{
+		Locker l(resetParentMutex);
+		listResetParent.push_back(child);
+	}
+	void resetParentList()
+	{
+		Locker l(resetParentMutex);
+		auto it = listResetParent.begin();
+		while (it != listResetParent.end())
+		{
+			(*it)->setParent(nullptr);
+			it = listResetParent.erase(it);
+		}
+	}
+	bool isInResetParentList(DisplayObject* d)
+	{
+		Locker l(resetParentMutex);
+		auto it = listResetParent.begin();
+		while (it != listResetParent.end())
+		{
+			if ((*it).getPtr()==d)
+				return true;
+			it++;
+		}
+		return false;
+	}
+	void removeFromResetParentList(DisplayObject* d)
+	{
+		Locker l(resetParentMutex);
+		auto it = listResetParent.begin();
+		while (it != listResetParent.end())
+		{
+			if ((*it).getPtr()==d)
+			{
+				listResetParent.erase(it);
+				break;
+			}
+			it++;
+		}
+	}
 };
 
 class ParseThread: public IThreadJob
